/* This file contains routines which act on bitboards. */



/* -------------------------------------------------------------- 
 * The following Routines set up the static masks which are used 
 * for various purposes.
 * --------------------------------------------------------------
 */

/* mask[x] has only the xth bit true */
void initBitboardMask(void)
  {
  bitboard one = 1;
  int i;

  for (i=0;i<64;i++)
    mask[i] = one << i;
  }

/* These masks are used to easily scan for double pawn moves */
void initPawnMasks(void) 
  {
  doublePawnMask[WHITE]=mask[24]|mask[25]|mask[26]|mask[27]|mask[28]|mask[29]|mask[30]|mask[31];
  doublePawnMask[BLACK]=mask[32]|mask[33]|mask[34]|mask[35]|mask[36]|mask[37]|mask[38]|mask[39];
  }

void initOtherMasks(void)
  {
  leftMask = ~(mask[0]|mask[8]|mask[16]|mask[24]|mask[32]|mask[40]|mask[48]|mask[56]);
  rightMask = ~(mask[7]|mask[15]|mask[23]|mask[31]|mask[39]|mask[47]|mask[55]|mask[63]);
  rowmask = mask[0]|mask[1]|mask[2]|mask[3]|mask[4]|mask[5]|mask[6]|mask[7];   
  }




/* ------------------------------------------------------------------- 
 * all of the functions which initialize other bitboards are in 
 * this section. These include the attack boards for all the pieces
 * as well as the initial states of the dynamic position boards.
 * -------------------------------------------------------------------
 */


/* sets up the bitboards from the current 8x8 position. Used when the
 * starting position is midgame and given by a FEN string 
 */
void setupBitboards(void)
  {
  int i, j;

  for (i=0;i<6;i++)
    {
    position[i][WHITE]=0;
    position[i][BLACK]=0;
    }

  for (i=0;i<8;i++)
    for (j=0;j<8;j++)
      if (pos.board[i][j].piece)
        position[pos.board[i][j].piece-1][pos.board[i][j].colour] |= mask[i*8+j];

  allpieces[WHITE] = position[PAWN-1][WHITE]|position[BISHOP-1][WHITE]|position[KNIGHT-1][WHITE]|position[ROOK-1][WHITE]|position[QUEEN-1][WHITE]|position[KING-1][WHITE];
  allpieces[BLACK] = position[PAWN-1][BLACK]|position[BISHOP-1][BLACK]|position[KNIGHT-1][BLACK]|position[ROOK-1][BLACK]|position[QUEEN-1][BLACK]|position[KING-1][BLACK];
  }

/* sets up the initial position bitboards */
void initBitboardPositions(void)
  {
  position[PAWN-1][WHITE] = mask[8]|mask[9]|mask[10]|mask[11]|mask[12]|mask[13]|mask[14]|mask[15]; 
  position[PAWN-1][BLACK] = mask[48]|mask[49]|mask[50]|mask[51]|mask[52]|mask[53]|mask[54]|mask[55];
  position[BISHOP-1][WHITE] = mask[2]|mask[5];
  position[BISHOP-1][BLACK] = mask[58]|mask[61];
  position[KNIGHT-1][WHITE] = mask[1]|mask[6];
  position[KNIGHT-1][BLACK] = mask[57]|mask[62];
  position[ROOK-1][WHITE] = mask[0]|mask[7];
  position[ROOK-1][BLACK] = mask[56]|mask[63];
  position[QUEEN-1][WHITE] = mask[3];
  position[QUEEN-1][BLACK] = mask[59];
  position[KING-1][WHITE] = mask[4];
  position[KING-1][BLACK] = mask[60];
  allpieces[WHITE] = position[PAWN-1][WHITE]|position[BISHOP-1][WHITE]|position[KNIGHT-1][WHITE]|position[ROOK-1][WHITE]|position[QUEEN-1][WHITE]|position[KING-1][WHITE];
  allpieces[BLACK] = position[PAWN-1][BLACK]|position[BISHOP-1][BLACK]|position[KNIGHT-1][BLACK]|position[ROOK-1][BLACK]|position[QUEEN-1][BLACK]|position[KING-1][BLACK];
  }

/* initializes the knight attack boards */
void initKnightAttacks(void)
  {
  int i;
  for (i=0;i<64;i++)
    {
    knightAttacks[i] = 0;
    if ((i%8>0) && (i<48)) knightAttacks[i] = knightAttacks[i]|mask[i+15];
    if ((i%8<7) && (i<48)) knightAttacks[i] = knightAttacks[i]|mask[i+17];
    if ((i%8<6) && (i<56)) knightAttacks[i] = knightAttacks[i]|mask[i+10];
    if ((i%8<6) && (i>7)) knightAttacks[i] = knightAttacks[i]|mask[i-6];
    if ((i%8<7) && (i>15)) knightAttacks[i] = knightAttacks[i]|mask[i-15];
    if ((i%8>0) && (i>15)) knightAttacks[i] = knightAttacks[i]|mask[i-17];
    if ((i%8>1) && (i>7)) knightAttacks[i] = knightAttacks[i]|mask[i-10];
    if ((i%8>1) && (i<56)) knightAttacks[i] = knightAttacks[i]|mask[i+6];
    }
  }

/* initializes the king attack boards */
void initKingAttacks(void)
  {
  int i;
  for (i=0;i<64;i++)
    {
    kingAttacks[i]=0;
    if (((i/8)==((i+1)/8))&&((i+1)<64)) kingAttacks[i] = kingAttacks[i]|mask[i+1];
    if (((i/8)==((i-1)/8))&&((i-1)>=0)) kingAttacks[i] = kingAttacks[i]|mask[i-1];
    if (((i/8)==(((i-9)/8)+1))&&((i-9)>=0)) kingAttacks[i] = kingAttacks[i]|mask[i-9];
    if (((i/8)==(((i-8)/8)+1))&&((i-8)>=0)) kingAttacks[i] = kingAttacks[i]|mask[i-8];
    if (((i/8)==(((i-7)/8)+1))&&((i-7)>=0)) kingAttacks[i] = kingAttacks[i]|mask[i-7];
    if (((i/8)==(((i+7)/8)-1))&&((i+7)<64)) kingAttacks[i] = kingAttacks[i]|mask[i+7];
    if (((i/8)==(((i+8)/8)-1))&&((i+8)<64)) kingAttacks[i] = kingAttacks[i]|mask[i+8];
    if (((i/8)==(((i+9)/8)-1))&&((i+9)<64)) kingAttacks[i] = kingAttacks[i]|mask[i+9];
    }
  }

/* initializes the straight sliding attack boards (rook, queen) */
void initStraightBoards(void)
  {
  int i,j, k;

  for (i=0;i<8;i++)
    {
    for (j=0;j<256;j++)
      {
      slideFriendAttacks[i][j]=0;
      slideEnemyAttacks[i][j]=0;
      /* i is the position of the rook on the row 
       * j is the state index for same coloured pieces */
      /* 1. check to the right of the piece */
      for (k=i+1; k<8; k++)
        {
	/* if there is a piece, we cannot go any further. If not, 
           we add it as a possible move.                       */
        if (j&mask[k])
          {
          slideEnemyAttacks[i][j]=slideEnemyAttacks[i][j]|mask[k];
          break;
	  }
        slideEnemyAttacks[i][j]=slideEnemyAttacks[i][j]|mask[k];
        slideFriendAttacks[i][j]=slideFriendAttacks[i][j]|mask[k];
	}
      /* 2. check to the left of the piece */
      for (k=i-1; k>= 0; k--)
	{
        if (j&mask[k])
	  {
          slideEnemyAttacks[i][j]=slideEnemyAttacks[i][j]|mask[k];
          break;
	  }
        slideEnemyAttacks[i][j]=slideEnemyAttacks[i][j]|mask[k];
        slideFriendAttacks[i][j]=slideFriendAttacks[i][j]|mask[k];
	}
      }
    }
  }

/* initialize the attack boards for the diagonal sliding pieces 
 * (bishop, queen)
 */
void initDiagonalBoards(void)
  {
  int i, j;
  int position;
  int square;
  int lengthLeft, lengthRight;

  for (i=0;i<64;i++)     /* Position of Piece on board */
    {
    for (j=0;j<256;j++)  /* state of pieces along diagonal */
      {
      friendlyUp[i][j]=friendlyDown[i][j]=enemyUp[i][j]=enemyDown[i][j]=0;
      /* 1. Compute upwards boards */
      /* find how many squares there are going down the diagonal */
      position = i;
      lengthLeft = 0;
      while (1)
        {
        position -= 9;
        /* if it's off the board or more than a row apart then there */
        /* is no more to do here and we move on.                     */
        if ((position <0) || (((position+9)/8-position/8)!=1))
          break;
        lengthLeft++;
	}
      /* find how many squares there are going up the diagonal */
      position = i;
      lengthRight = 0;
      while (1)
	{
        position += 9;
        if ( (position > 63) || ((position/8-(position-9)/8)!=1))
          break;
        lengthRight++;
	}
      square = lengthLeft-1;     /* which bit of the index to consider */
      /* now cycle down and update the upwards boards */
      while (square>=0)
        {
	if (j&mask[square])       /* there is a piece here */
          {
          enemyUp[i][j]=enemyUp[i][j]|mask[i-9*(lengthLeft-square)];
          break;
          }
        /* no piece there means it's a possible move in both cases */
        enemyUp[i][j]=enemyUp[i][j]|mask[i-9*(lengthLeft-square)];
        friendlyUp[i][j]=friendlyUp[i][j]|mask[i-9*(lengthLeft-square)];
        square--;
	}
      /* next cycle up and update the upwards boards */
      square = lengthLeft+1;
      while (square <= lengthLeft+lengthRight)
	{
        if (j&mask[square])      
          {
          enemyUp[i][j]=enemyUp[i][j]|mask[i+9*(square-lengthLeft)];
          break;
	  }
        enemyUp[i][j]=enemyUp[i][j]|mask[i+9*(square-lengthLeft)];
        friendlyUp[i][j]=friendlyUp[i][j]|mask[i+9*(square-lengthLeft)];
        square++;
	}
      /* 2. Compute Downwards bitboards */
      /* first, count the lengths in each direction from the piece */
      lengthLeft=lengthRight=0;
      /* count down (right) first */
      position=i;
      while (1)
	{
        position -= 7;
        if ( (position < 0) || (((position+7)/8-position/8)!=1))
          break;
        lengthRight++;
	}
      /* next count up (left) */
      position=i;
      while (1)
	{
        position += 7;
        if ((position > 63) || (((position)/8-(position-7)/8)!=1))
          break;
        lengthLeft++;
	}
      /* now cycle down (right) and update the downwards bitboards */
      square = lengthRight-1;
      while (square >= 0)
	{
        if (j&mask[square])       /* there is a piece here */
          {
          enemyDown[i][j]=enemyDown[i][j]|mask[i-7*(lengthRight-square)];
          break;
          }
        enemyDown[i][j]=enemyDown[i][j]|mask[i-7*(lengthRight-square)];
        friendlyDown[i][j]=friendlyDown[i][j]|mask[i-7*(lengthRight-square)];
        square--;
	}
      /* lastly, cycle up (left) and update the downwards bitboards */
      square = lengthRight+1;
      while (square <= lengthLeft+lengthRight)
	{
        if (j&mask[square])       /* there is a piece here */
          {
          enemyDown[i][j]=enemyDown[i][j]|mask[i+7*(square-lengthRight)];
          break;
          }
        enemyDown[i][j]=enemyDown[i][j]|mask[i+7*(square-lengthRight)];
        friendlyDown[i][j]=friendlyDown[i][j]|mask[i+7*(square-lengthRight)];
        square++;
	}
      }
    }
  }

/* This function just ties all the initializations together, and
 * is called upon program startup
 */
void initBitboards(void)
  {
  initBitboardMask();
  initPawnMasks();
  initOtherMasks();
  initBitboardPositions();
  initKnightAttacks();
  initKingAttacks();
  initStraightBoards();
  initDiagonalBoards();
  }



/* ----------------------------------------------------------------
 * These are the bitboard parsers which turn attack boards into 
 * move lists. There is one specifically for pawns because their
 * attack boards are generated all at once, instead of one piece
 * at a time. This can be done because it is rather easy to tell
 * where the pawn came from just by knowing which kind of move
 * you are trying to generate, and pawns have only 4 options.
 * These routines can work in 2 ways: Pseudolegal, which doesn't
 * check to see if the side to move will be in check afterwards
 * (which is much faster), and legal, which does check. When
 * the pseudolegal moves are generated, the legality checking is
 * postponed until they are just about to be searched.
 * ----------------------------------------------------------------
 */

/* takes a from square and a bitboard and generates a move list.
   User must ensure that the list is initialized.
   Mode 0 doesn't check for check. Mode 1 does.
 */
void parseBitboard (int from, struct MoveList *ml, bitboard attack, int mode)
  {
  struct Move tempmove;
  int i;

  if (!attack) return;     /* There are no moves */

  tempmove.promote = EMPTY;
  
  /* first,... which square is the piece coming from? */
  tempmove.from_column = from%8;
  tempmove.from_row = from/8;
  
  /* now scan through and find target squares */
  for (i=0; i<64; i++)
    {
    if (attack&mask[i])
      {
      tempmove.to_row = i/8;
      tempmove.to_column = i%8;
      tempmove.piece = pos.board[tempmove.to_row][tempmove.to_column].piece;
      if (mode==1)
	{
        if (!will_check(tempmove))
	  {
          if (tempmove.piece!=EMPTY) addMove(tempmove, ml);
          else appendMove(tempmove, ml);
	  }
	}
      else
        {
        if (tempmove.piece!=EMPTY) addMove(tempmove, ml);
        else appendMove(tempmove, ml);
	}
      }
    }      
  }

/* type 1: 1 move forward
 * type 2: 2 squares forward
 * type 3: kill left
 * type 4: kill right
 */
void parsePawnBoard (int type, struct MoveList *ml, bitboard attack, int mode)
  {
  struct Move tempmove;
  int i;

  if (!attack) return;      /* there are no moves */

  for (i=0;i<64;i++)
    {
    if (attack&mask[i])
      {
      tempmove.to_row = i/8;
      tempmove.to_column = i%8;
      tempmove.piece = pos.board[tempmove.to_row][tempmove.to_column].piece;
      if (type==1)
        {
        if (pos.who_moves == WHITE)
          {
	  tempmove.from_row = tempmove.to_row-1;
          tempmove.from_column = tempmove.to_column;
	  }
        else 
          {
          tempmove.from_row = tempmove.to_row+1;
          tempmove.from_column = tempmove.to_column;
	  }
	}
      else if (type==2)
	{
        if (pos.who_moves == WHITE)
	  {
          tempmove.from_row = tempmove.to_row-2;
          tempmove.from_column = tempmove.to_column;
	  }
        else
          {
          tempmove.from_row = tempmove.to_row+2;
          tempmove.from_column = tempmove.to_column;
	  }
	}
      else if (type==3)     /* left kill */
        {
        if (pos.who_moves == WHITE)
	  {
          tempmove.from_row = tempmove.to_row-1;
          tempmove.from_column = tempmove.to_column+1;
	  }
        else 
	  {
          tempmove.from_row = tempmove.to_row+1;
          tempmove.from_column = tempmove.to_column+1;
	  }
	}
      else if (type==4)     /* right kill */
        {
        if (pos.who_moves == WHITE)
	  {
          tempmove.from_row = tempmove.to_row -1;
          tempmove.from_column = tempmove.to_column-1;
	  }
        else
	  {
          tempmove.from_row = tempmove.to_row+1;
          tempmove.from_column = tempmove.to_column-1;
	  }
	}
      if ((mode==1)&&(will_check(tempmove)))
        {/*we don't add anything, because it leads to check */}
      else
	{
        /* Add the promotion Cases here */
        if (((pos.who_moves==WHITE)&&(tempmove.to_row==7))||((pos.who_moves==BLACK)&&(tempmove.to_row==0)))
          {
          tempmove.promote=BISHOP; addMove(tempmove, ml);
          tempmove.promote=KNIGHT; addMove(tempmove, ml);
          tempmove.promote=ROOK;   addMove(tempmove, ml);
          tempmove.promote=QUEEN;  addMove(tempmove, ml);
	  }
        else
          {
          tempmove.promote=EMPTY;
	  if (type==1||type==2) appendMove(tempmove, ml);  /* non captures to the end! */
          else addMove(tempmove, ml);
	  }
	}
      }
    }
  }



/* ----------------------------------------------------------
 * These are all state routines which figure out the indexes
 * to the attack boards.
 * ----------------------------------------------------------
 */

/* returns the upwards diagonal (a1-h8) state index */
int upState(int colour, int location)
  {
  int index = 0;
  int position = location;
  int spot = 0;

  /* find the first (leftmost) square of the diagonal */
  while (1)
    {
    position -= 9;
    if ( (position<0) || (((position+9)/8 - position/8)!=1))
      {
      position +=9;
      break;
      }
    }
  //if (location==54) printf("Position is %d\n",position);

  /* now cycle up the diagonal and calculate the index along the way */
  if (allpieces[colour]&mask[position])
    index=index|mask[spot];
  spot++;
  position+=9;
  while ((position < 64) && (((position/8-(position-9)/8)==1)))
    {
    if (allpieces[colour]&mask[position])   /* there is a piece here */
      index=index|mask[spot];
    spot++;
    position+=9;
    }
  return index;
  }

/* returns the downward diagonal (a8-h1) state index */
int downState(int colour, int location)
  {
  int index = 0;
  int position = location;
  int initial;
  int spot = 0;

  /* find the first (rightmost) square of the diagonal */
  while (1)
    {
    position -= 7;
    if ( (position<0) || (((position+7)/8 - position/8)!=1))
      {
      position +=7;
      break;
      }
    }

  initial=position;
  /* now cycle up the diagonal and calculate the index along the way */
  while ((initial==position)||((position < 63) && (((position/8-(position-7)/8)==1) || ((position<8)&&((position/8-(position-7)/8)==0)))))
    {
    if (allpieces[colour]&mask[position])   /* there is a piece here */
      index=index|mask[spot];
    spot++;
    position+=7;
    }
  return index;
  }

/* returns the vertical state for the given colour of the given column */
int verticalState (int colour, int column)
  {
  int state = 0;
  int i;

  for (i=0;i<8;i++)
    {
    if (allpieces[colour]&mask[i*8+column])
      state=state|mask[i];
    }
  return state;
  }

/* transforms the first row of the source board into the given column 
 * (essentially the opposite of above)
 */
bitboard rotate90 (bitboard source, int column)
  {
  bitboard target = 0;
  int i;

  for (i=0;i<8;i++)
    {
    if (source&mask[i])
    target=target|mask[i*8+column];
    }
  return target;
  }



/* -------------------------------------------------------------
 * These are the actual move generation routines, one for each
 * piece type. They generate ALL moves,.. with the same modes
 * as the board parsers above. The position of the piece must
 * be passed.
 * -------------------------------------------------------------
 */

/* generates knight moves for given colour */
void genKnightMoves(struct MoveList *ml, int mode, int i)
  {
  parseBitboard(i, ml, knightAttacks[i]&~allpieces[pos.who_moves], mode);
  }

/* generates the king moves for the given colour */
void genKingMoves(struct MoveList *ml, int mode, int i)
  {
  struct Move tempmove;

  parseBitboard(i, ml, kingAttacks[i]&~allpieces[pos.who_moves], mode);

  /* Must consider Castling moves here */
  if (pos.who_moves == WHITE && i==4)
    {
      /* king side */
    if (!pos.wKing&&!pos.wkRook)
      if ((pos.board[0][5].piece==EMPTY)&&(pos.board[0][6].piece==EMPTY))
        if (!check_for_attack(0, 4, BLACK))
          if (!check_for_attack(0, 5, BLACK))
            {
            tempmove.piece=tempmove.promote=EMPTY;
            tempmove.to_row=tempmove.from_row=0;
            tempmove.from_column=4;
            tempmove.to_column=6;
            if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
              addMove(tempmove, ml);
	    }
    /* queen side */
    if (!pos.wKing&&!pos.wqRook)
      if ((pos.board[0][3].piece==EMPTY)&&(pos.board[0][2].piece==EMPTY)&&(pos.board[0][1].piece==EMPTY))
        if (!check_for_attack(0, 4, BLACK))
          if (!check_for_attack(0, 3, BLACK))
            {
            tempmove.piece=tempmove.promote=EMPTY;
            tempmove.to_row=tempmove.from_row=0;
            tempmove.from_column = 4;
            tempmove.to_column=2;
            if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
              addMove(tempmove, ml);
            }
    }
  else if (pos.who_moves==BLACK && i==60)  /* black */
   {
    /* king side */
    if (!pos.bKing&&!pos.bkRook)
      if ((pos.board[7][5].piece==EMPTY)&&(pos.board[7][6].piece==EMPTY))
        if (!check_for_attack(7, 4, WHITE))
          if (!check_for_attack(7, 5, WHITE))
            {
            tempmove.piece=tempmove.promote=EMPTY;
            tempmove.to_row=tempmove.from_row=7;
            tempmove.from_column=4;
            tempmove.to_column=6;
            if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
              addMove(tempmove, ml);
	    }
    /* queen side */
    if (!pos.bKing&&!pos.bqRook)
      if ((pos.board[7][3].piece==EMPTY)&&(pos.board[7][2].piece==EMPTY)&&(pos.board[7][1].piece==EMPTY))
        if (!check_for_attack(7, 4, WHITE))
          if (!check_for_attack(7, 3, WHITE))
            {
            tempmove.piece=tempmove.promote=EMPTY;
            tempmove.to_row=tempmove.from_row=7;
            tempmove.from_column = 4;
            tempmove.to_column=2;
            if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
              addMove(tempmove, ml);
            }
    }  
  }
  
/* generates the rook moves for the given colour */
void genRookMoves(struct MoveList *ml, int mode, int i)
  {
  int row, column;
  bitboard horattacks, horattacks1, horattacks2 = 0;
  bitboard verattacks, verattacks1, verattacks2 = 0;

  /* 1. Find the horizontal moves */
  row = i/8;
  column = i%8;
  horattacks1 = slideFriendAttacks[column][(allpieces[pos.who_moves]>>(row*8))&rowmask];
  horattacks2 = slideEnemyAttacks[column][(allpieces[!pos.who_moves]>>(row*8))&rowmask];
  horattacks= (horattacks1&horattacks2)<<(row*8);
  /* 2. Find the vertical moves */
  verattacks1 = slideFriendAttacks[row][verticalState(pos.who_moves, column)];
  verattacks2 = slideEnemyAttacks[row][verticalState(!pos.who_moves, column)];
  verattacks = rotate90(verattacks1&verattacks2, column);
  parseBitboard(i, ml, verattacks|horattacks, mode);
  } 

void genBishopMoves (struct MoveList *ml, int mode, int i)
  {
  bitboard upattacks, downattacks;

  upattacks = friendlyUp[i][upState(pos.who_moves, i)]&enemyUp[i][upState(!pos.who_moves, i)];
  downattacks = friendlyDown[i][downState(pos.who_moves, i)]&enemyDown[i][downState(!pos.who_moves, i)];
      
  //printf("position %d upf %d upe %d downf %d downe %d\n",i,upState(pos.who_moves, i),upState(!pos.who_moves,i),downState(pos.who_moves,i),downState(!pos.who_moves, i));
  //drawBitboard(friendlyUp[i][upState(pos.who_moves, i)]);
      
  parseBitboard(i, ml, upattacks|downattacks, mode);
  }

void genQueenMoves(struct MoveList *ml, int mode, int i)
  {
  int row, column;
  bitboard horattacks, horattacks1, horattacks2 = 0;
  bitboard verattacks, verattacks1, verattacks2 = 0;
  bitboard upattacks, downattacks;

  /* 1. Find the horizontal moves */
  row = i/8;
  column = i%8;
  horattacks1 = slideFriendAttacks[column][(allpieces[pos.who_moves]>>(row*8))&rowmask];
  horattacks2 = slideEnemyAttacks[column][(allpieces[!pos.who_moves]>>(row*8))&rowmask];
  horattacks= (horattacks1&horattacks2)<<(row*8);
  /* 2. Find the vertical moves */
  verattacks1 = slideFriendAttacks[row][verticalState(pos.who_moves, column)];
  verattacks2 = slideEnemyAttacks[row][verticalState(!pos.who_moves, column)];
  verattacks = rotate90(verattacks1&verattacks2, column);
  /* 3. Find the diagonal moves */
  upattacks = friendlyUp[i][upState(pos.who_moves, i)]&enemyUp[i][upState(!pos.who_moves, i)];
  downattacks = friendlyDown[i][downState(pos.who_moves, i)]&enemyDown[i][downState(!pos.who_moves, i)];
  
  parseBitboard(i, ml, upattacks|downattacks|verattacks|horattacks, mode);
  }

/* Black and White pawns move differently, and so have different cases */
void genPawnMoves(struct MoveList *ml, int mode)
  {
  bitboard oneahead;
  bitboard twoahead;
  bitboard killleft,killright;
  bitboard material = allpieces[WHITE]|allpieces[BLACK];
  struct Move tempmove;

  if (!position[PAWN-1][pos.who_moves]) return;   /* no pawns */

  if (pos.who_moves==WHITE)
    {
    oneahead = (position[PAWN-1][WHITE]<<8)&~material;
    parsePawnBoard(1, ml, oneahead, mode);
    twoahead = (oneahead<<8)&~material&doublePawnMask[WHITE];
    parsePawnBoard(2, ml, twoahead, mode);
    killleft = (position[PAWN-1][WHITE]<<7)&allpieces[BLACK]&rightMask;
    parsePawnBoard(3, ml, killleft, mode);
    killright = (position[PAWN-1][WHITE]<<9)&allpieces[BLACK]&leftMask;
    parsePawnBoard(4, ml, killright, mode);
    /* check enpassant case */
    if (pos.ippawn != -1)      /* there is an eligible pawn */
      if (pos.ippawn > 0)     /* 1. kill to right */
        {
        if ((pos.board[4][pos.ippawn-1].piece==PAWN)&&(pos.board[4][pos.ippawn-1].colour==WHITE))
          {
          tempmove.promote = EMPTY; tempmove.piece=ENPASSENTPAWN;
          tempmove.from_row=4; tempmove.to_row=5;
          tempmove.from_column=pos.ippawn-1; tempmove.to_column=pos.ippawn;
          if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
            addMove(tempmove, ml);
	  }
	}
      else if (pos.ippawn < 7)    /* 2. kill to left */
        {
        if ((pos.board[4][pos.ippawn+1].piece==PAWN)&&(pos.board[4][pos.ippawn+1].colour==WHITE))
	  {
          tempmove.promote=EMPTY; tempmove.piece=ENPASSENTPAWN;
          tempmove.from_row=4; tempmove.to_row=5;
          tempmove.from_column=pos.ippawn+1; tempmove.to_column=pos.ippawn;
          if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
            addMove(tempmove, ml);
	  }
	}
    }
  else
    {
    oneahead = (position[PAWN-1][BLACK]>>8)&~material;
    parsePawnBoard(1, ml, oneahead, mode);
    twoahead = (oneahead>>8)&~material&doublePawnMask[BLACK];
    parsePawnBoard(2, ml, twoahead, mode);
    killleft = (position[PAWN-1][BLACK]>>9)&allpieces[WHITE]&rightMask;
    parsePawnBoard(3, ml, killleft, mode);
    killright = (position[PAWN-1][BLACK]>>7)&allpieces[WHITE]&leftMask;
    parsePawnBoard(4, ml, killright, mode);
    /* check enpassant case */
    if (pos.ippawn != -1)      /* there is an eligible pawn */
      if (pos.ippawn > 0)     /* 1. kill to right */
        {
        if ((pos.board[3][pos.ippawn-1].piece==PAWN)&&(pos.board[3][pos.ippawn-1].colour==BLACK))
          {
          tempmove.promote = EMPTY; tempmove.piece=ENPASSENTPAWN;
          tempmove.from_row=3; tempmove.to_row=2;
          tempmove.from_column=pos.ippawn-1; tempmove.to_column=pos.ippawn;
          if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
            addMove(tempmove, ml);
	  }
	}
      else if (pos.ippawn < 7)    /* 2. kill to left */
        {
        if ((pos.board[3][pos.ippawn+1].piece==PAWN)&&(pos.board[3][pos.ippawn+1].colour==BLACK))
	  {
          tempmove.promote=EMPTY; tempmove.piece=ENPASSENTPAWN;
          tempmove.from_row=3; tempmove.to_row=2;
          tempmove.from_column=pos.ippawn+1; tempmove.to_column=pos.ippawn;
          if ((mode==0)||((mode==1)&&(!will_check(tempmove))))
            addMove(tempmove, ml);
	  }
	}
    }
  }


/* Incorporates all the above functions into one. Should be faster
 * because we only need to loop though to find pieces once...
 */
void genMoves(struct MoveList *ml, int mode)
  {
  int i;
  clock_t startTime, endTime;
  startTime = getMS();
  if (ml->size > 0)
    clearList(ml);
  initList(ml);

  genPawnMoves(ml, mode);
  if (pos.who_moves==WHITE) genKingMoves(ml,mode,pos.wKingRow*8+pos.wKingColumn);
  else  genKingMoves(ml, mode, pos.bKingRow*8+pos.bKingColumn);
  
  for (i=0;i<64;i++)
    {
    if (allpieces[pos.who_moves]&mask[i])
      {
      if (position[KNIGHT-1][pos.who_moves]&mask[i])
        genKnightMoves(ml, mode, i);
      else if (position[BISHOP-1][pos.who_moves]&mask[i])
        genBishopMoves(ml, mode, i);
      else if (position[ROOK-1][pos.who_moves]&mask[i])
        genRookMoves(ml, mode, i);
      else if (position[QUEEN-1][pos.who_moves]&mask[i])
        genQueenMoves(ml, mode, i);
      }
    }
  endTime=getMS();
  timeGenerating+=(double)(endTime-startTime)/(double)1000;
  }


/* ---------------------------------------------------------------
 * These are other miscellaneous bitboard routines.
 * ---------------------------------------------------------------
 */





/* -----------------------------------------------------------------
 * The rest of the routines in this file are for debugging purposes
 * and don't have any effect on overall performance
 * -----------------------------------------------------------------
 */

/* Prints out the list of all possible moves */
void testBitboard(int mode)
  {
  struct MoveList ml;
  initList(&ml);
  genMoves(&ml, mode);
  printf("(%d moves available)\n",ml.size);
  printMoveList(&ml);
  clearList(&ml);
  }

/* This compares the dynamic bitboards and the 8x8 position matrix.
 * returns TRUE is they're the same and FALSE otherwise.
 */
int compareBoards(void)
  {
  int piece ,colour;
  int i;

  for (piece=0; piece<6; piece++)
    for (colour=0; colour<2; colour++)
      for (i=0;i<64;i++)
        if (position[piece][colour]&mask[i])
          if ((pos.board[i/8][i%8].piece!=piece+1)||(pos.board[i/8][i%8].colour!=colour))
            return FALSE;
  return TRUE;
  }

/* Compares the indivdual bitboards to the allpieces bitboards.
 */
void compBitboards(void)
  {
  bitboard allWhite=position[PAWN-1][WHITE]|position[BISHOP-1][WHITE]|position[KNIGHT-1][WHITE]|position[ROOK-1][WHITE]|position[QUEEN-1][WHITE]|position[KING-1][WHITE];
  bitboard allBlack=position[PAWN-1][BLACK]|position[BISHOP-1][BLACK]|position[KNIGHT-1][BLACK]|position[ROOK-1][BLACK]|position[QUEEN-1][BLACK]|position[KING-1][BLACK];

  drawBitboard(allWhite);
  drawBitboard(allpieces[WHITE]);
  drawBitboard(allBlack);
  drawBitboard(allpieces[BLACK]);

  if (allpieces[WHITE]==allWhite)
    printf("All White Pieces is EQUAL\n");
  else
    printf("All White Pieces is FUCKED UP!\n");
  if (allpieces[BLACK]==allBlack)
    printf("All Black Pieces is EQUAL\n");
  else
    printf("All Black Pieces is FUCKED UP!\n");
  }

/* Compares the indivdual bitboards to the allpieces bitboards.
 * returns true/false                                           */
int compBit(void)
  {
  bitboard allWhite=position[PAWN-1][WHITE]|position[BISHOP-1][WHITE]|position[KNIGHT-1][WHITE]|position[ROOK-1][WHITE]|position[QUEEN-1][WHITE]|position[KING-1][WHITE];
  bitboard allBlack=position[PAWN-1][BLACK]|position[BISHOP-1][BLACK]|position[KNIGHT-1][BLACK]|position[ROOK-1][BLACK]|position[QUEEN-1][BLACK]|position[KING-1][BLACK];

  if (allpieces[WHITE]!=allWhite)
    return FALSE;
  else if (allpieces[BLACK]!=allBlack)
    return FALSE;
  return TRUE;
  }

void showBitboards(void)
  {
  int i, piece, colour;

  for (piece=0; piece<6; piece++)
    for (colour=0; colour<2; colour++)
      {
      printf("\np: %d c: %d\n", piece, colour);
      for (i=0; i<64; i++)
        {
        if ((!(i%8))&&(i!=0)) printf("\n");
        printf("%i ",(position[piece][colour]&mask[i])>>i);
        }  
      }
  printf("\nAll Pieces Black:\n");
  drawBitboard(allpieces[BLACK]);
  printf("All Pieces White:\n");
  drawBitboard(allpieces[WHITE]);

  printf("\n");
  }

/* tests the speeds of the respective move generation routines */
void testGeneration(void)
  {
  struct MoveList ml;
  clock_t startTime, endTime;
  double genTime;
  int i;

  initList(&ml);
  startTime = getMS();
  for (i=0; i<2000; i++)
    {
    clearList(&ml);
    generateMoves(&ml, 0);
    }
  endTime = getMS();
  genTime=(double)(endTime-startTime)/(double)1000;  /* in seconds */
  printf("8x8 generation found %d moves in %2.2fs (for 2000).\n",ml.size, genTime);

  clearList(&ml);
  startTime = getMS();
  for (i=0; i<2000; i++)
    {
    clearList(&ml);
    genMoves(&ml, 0);
    }
  endTime=getMS();
  genTime=(double)(endTime-startTime)/(double)1000;
  printf("Bitboard generation found %d moves in %2.2fs (for 2000).\n", ml.size, genTime);
  
  clearList(&ml);
  startTime=getMS();
  for (i=0; i<2000; i++)
    {
    clearList(&ml);
    genMoves(&ml, 1);
    }
  endTime=getMS();
  genTime=(double)(endTime-startTime)/(double)1000;
  printf("Bitboard gen w/ check found %d moves in %2.2fs (for 2000).\n", ml.size, genTime);
  
  clearList(&ml);
  }

/* draws the given bitboard to stdout - for debugging purposes */
void drawBitboard (bitboard source)
  {
  int i;

  for (i=0; i<64; i++)
    {
    if ((!(i%8))&&(i!=0)) printf("\n");
    printf("%i ",(source&mask[i])>>i);
    }
  printf("\n\n");
  }  





