/* These are the routines used to access and edit the book file  */




/* Tests that the book file exists and is readable.
 * returns true/false
 */
int testBook (void)
  {
  FILE *fp;

  if ((fp = fopen(bookfile,"rb")) == NULL)
    return FALSE;
  fclose(fp);
  return TRUE;
  }

/* Erase the book file, create a new one and add the first entry (e4) */
void initBookFile (void)
  {
  FILE *fp;
  struct BookMove firstmove;

  firstmove.move.from_row = 1;
  firstmove.move.from_column = 4;
  firstmove.move.to_row = 3;
  firstmove.move.to_column = 4;
  firstmove.move.piece = EMPTY;
  firstmove.move.promote = EMPTY;
  firstmove.me = 0;        /* the root entry */
  firstmove.next = -1;     /* the rest is empty */
  firstmove.up = -1;
  firstmove.value = 0;

  fp = fopen(bookfile,"wb");
  fwrite(&firstmove, sizeof(struct BookMove), 1, fp);
  fclose(fp);
  }

/* moves is the games history thus far, it will fill up options with
 * all the possible book moves if there are any. It returns FALSE if there
 * are no options or TRUE otherwise. The BookMoveList will be empty if it
 * returns FALSE
 */
int bookSearch(struct MoveList moves, struct BookMoveList *options)
  {
  struct MoveList templist;
  struct Move current;
  struct BookMove fileMove;
  FILE *fp;

  /* first we need to reverse the order of the moves list */
  initList(&templist);
  reverseList(moves, &templist);
  clearBookList(options);

  fp = fopen(bookfile, "rb");

  while(templist.size != EMPTY)
    {
    moveCopy(getMove(templist), &current);

    /* now we need to find this move in the "next" sequence of moves in
     * the file. If we get to the end of this sequence without finding it,
     * then we return FALSE
     */
    while(1)
      {
      fread(&fileMove, sizeof(struct BookMove), 1, fp);
      if ((fileMove.move.from_row == current.from_row)&&(fileMove.move.from_column == current.from_column)&&(fileMove.move.to_row == current.to_row)&&(fileMove.move.to_column == current.to_column))
        if (fileMove.up != -1)
          {
          fseek(fp, fileMove.up, SEEK_SET);
          break;
          }
        else               /* no more moves */
          {
          clearList(&templist);
          fclose(fp);
          return FALSE;
          }
      else if (fileMove.next == -1)
        {
        clearList(&templist);
        fclose(fp);
        return FALSE;
        }
      else
        fseek(fp, fileMove.next, SEEK_SET);
      }
    deleteMove(&templist);
    }

  /* now we fill up the BookMoveList with the "next" sequence of moves
   * from this position in the file and return TRUE
   */

  while(1)
    {
    fread(&fileMove, sizeof(struct BookMove), 1, fp);
    addBookMove(fileMove, options);
    if (fileMove.next == -1)
      break;
    else
      fseek(fp, fileMove.next, SEEK_SET);
    }
  fclose(fp);
  return TRUE;
  }


/* This function takes in a bookMoveList and prints in out to the
 * screen (for drawing/testing purposes)
 */
void printBookList(struct BookMoveList *options)
  {
  int i;
  struct BookMoveListNode *current;
  struct Move mov;
  char output[10];

  current = options->head;

  //printf("\tThe Book Moves from this Position:\n");
  for (i=0;i<options->size;i++)
    {
    moveCopy(current->bmov.move, &mov);
    createAlgebraOutput(mov, output);
    //printf("\t%-8s score=%d me=%ld next=%ld up=%ld\n",output, current->bmov.value, current->bmov.me, current->bmov.next, current->bmov.up);
    printf("\t%-8s score=%d\n",output,current->bmov.value);
    current = current -> next;
    }
  }

/* moves is the game history thus far, it will go through the book
 * file and add any moves which don't already exist. Returns the #
 * of moves added to the file.
 */
int bookUpdate(struct MoveList moves)
  {
  struct MoveList templist;
  struct Move current;
  struct BookMove fileMove;
  struct BookMove newMove;
  FILE *fp;
  int number = 0;

  /* first we need to reverse the order of the moves list */
  initList(&templist);
  reverseList(moves, &templist);
  
  fp = fopen(bookfile, "rb+");

  //printf("File Opened. His Sz %d.\n",templist.size);

  while(templist.size != EMPTY)
    {
      //printf("%d moves left\n",templist.size);
    moveCopy(getMove(templist), &current);

    /* now we need to find this move in the "next" sequence of moves in
     * the file. If we get to the end of this sequence without finding it,
     * then we add it and the remaining moves (the remaining moves can't
     * possibly exist once we get one which doesn't)
     */
    while(1)
      {
      fread(&fileMove, sizeof(struct BookMove), 1, fp);
      //printf("up=%ld next=%ld me=%ld\n",fileMove.up,fileMove.next,fileMove.me);
      
      if ((fileMove.move.from_row == current.from_row) && (fileMove.move.from_column == current.from_column) && (fileMove.move.to_row == current.to_row)&&(fileMove.move.to_column == current.to_column))
        {
        if (fileMove.up == -1)
          {
	    //printf("Move Found, can't go up. Adding....\n");
          /* the next record doesn't exist, so we read it in and add it */
          /* Once one of the moves isn't found, we are guaranteed that the
             rest won't be found as well, so we add all the rest.       */
          deleteMove(&templist);
          while(templist.size)
            {
	      //printf("Adding,... %d left\n",templist.size);
            moveCopy(getMove(templist), &current);
            newMove.move.from_row = current.from_row;
            newMove.move.from_column = current.from_column;
            newMove.move.to_row = current.to_row;
            newMove.move.to_column = current.to_column;
            newMove.move.piece = current.piece;
            newMove.move.promote = current.promote;
            newMove.next = -1;
            newMove.up = -1;
            fseek(fp, 0, SEEK_END);
            newMove.me = ftell(fp);
            fileMove.up = newMove.me;
            newMove.value = 0;
            /* now we have update the info for both records and we need to
               write both of them to the file                         */
            fwrite(&newMove, sizeof(struct BookMove), 1, fp);
            fseek(fp, fileMove.me, SEEK_SET);
            fwrite(&fileMove, sizeof(struct BookMove), 1, fp);
            /* we need to have the right contents in fileMove */
            fseek(fp, newMove.me, SEEK_SET);
            fread(&fileMove, sizeof(struct BookMove),1,fp);
            deleteMove(&templist);
            number++;
            }
          fclose(fp);
          //printf("File Closed\n");
          return number;
          }
        else
          {
	    //printf("move found: moving up\n");
          fseek(fp, fileMove.up, SEEK_SET);
          break;
          }
        }
      else if (fileMove.next == -1)
        {
	  //printf("Not Found,... end of list. Adding...\n");
        /* we need to add the current move to the list because
           it doesn't exist. We then add all the remaining moves
           because they aren't in the file either                 */

        /* first add this move to the "next" sequence    */
        newMove.move.from_row = current.from_row;
        newMove.move.from_column = current.from_column;
        newMove.move.to_row = current.to_row;
        newMove.move.to_column = current.to_column;
        newMove.move.piece = current.piece;
        newMove.move.promote = current.promote;
        newMove.next = -1;
        newMove.up = -1;
        fseek(fp, 0, SEEK_END);
        newMove.me = ftell(fp);
        fileMove.next = newMove.me;
        newMove.value = 0;
        /* now we have updated the info for both records and we need to
           write both of them to the file                         */
        fwrite(&newMove, sizeof(struct BookMove), 1, fp);
        fseek(fp, fileMove.me, SEEK_SET);
        fwrite(&fileMove, sizeof(struct BookMove), 1, fp);
        /* we need to have the right contents in fileMove */
        fseek(fp, newMove.me, SEEK_SET);
        fread(&fileMove, sizeof(struct BookMove),1,fp);
        deleteMove(&templist);
        number++;

        /* now add the rest of the moves to the "up" sequence */
        while(templist.size)
          {
	    //printf("Adding,... %d left\n",templist.size);
          moveCopy(getMove(templist), &current);
          newMove.move.from_row = current.from_row;
          newMove.move.from_column = current.from_column;
          newMove.move.to_row = current.to_row;
          newMove.move.to_column = current.to_column;
          newMove.move.piece = current.piece;
          newMove.move.promote = current.promote;
          newMove.next = -1;
          newMove.up = -1;
          fseek(fp, 0, SEEK_END);
          newMove.me = ftell(fp);
          fileMove.up = newMove.me;
          newMove.value = 0;
          /* now we have update the info for both records and we need to
             write both of them to the file                         */
          fwrite(&newMove, sizeof(struct BookMove), 1, fp);
          fseek(fp, fileMove.me, SEEK_SET);
          fwrite(&fileMove, sizeof(struct BookMove), 1, fp);
          /* we need to have the right contents in fileMove */
          fseek(fp, newMove.me, SEEK_SET);
          fread(&fileMove, sizeof(struct BookMove),1,fp);
          deleteMove(&templist);
          number++;
          }
        fclose(fp);
        return number;
        }
      else
        fseek(fp, fileMove.next, SEEK_SET);
      }
    deleteMove(&templist);
    }
  fclose(fp);     /* nothing added if we get here (all moves were found) */
  return number;
  }

void showBook(struct MoveList gameHistory)
  {
  struct BookMoveList bookMoves;
 
  initBookList(&bookMoves);     /* initialize the book moves list */

  if (BOOKMODE)
    {
    bookSearch(gameHistory, &bookMoves);
    printf("\tThere are %d moves in the book\n",bookMoves.size);
    if (bookMoves.size)
      printBookList(&bookMoves);
    }
  else
    printf("\t* You must have the book enabled to view it's contents\n");

  printf("\n");
  fflush(stdout);
  }







