/* These are the general Chess Routines which make moves, ect... */



/* Checks to see if the game has reached the endgame stage. Returns true/false */
int checkForEndgame (void)
{
  /* The endgame has commenced when there are less than 5 pieces on either side */
  if (stats.num_black_pieces < 5)
	return TRUE;
  else if (stats.num_white_pieces < 5)
	return TRUE;
  else
	return FALSE;
}

/* Check to see who's winning (based on material strength) */
int whosWinning(void)
{
  if (stats.white_strength > stats.black_strength)
	return WHITE;
  else if (stats.white_strength < stats.black_strength)
	return BLACK;
  else if (stats.num_white_pieces < stats.num_black_pieces)
	return BLACK;
  else
	return WHITE;     /* Default */
}

/* This function checks to see if the game is finished and outputs the
 * appropriate message to the screen.
 */
int checkEndgame (int who_moves)
  {
  if ((stats.num_black_moves == 0) && (stats.black_in_check) && (who_moves == BLACK))
    { printf("1-0 {White Mates}\n"); fflush(stdout); return TRUE; }   /* black is mated */
  else if ((stats.num_black_moves == 0) && (!stats.black_in_check) && (who_moves == BLACK))
    { printf("1/2-1/2 {Stalemate}\n"); fflush(stdout); return TRUE; } /* Black is stalemated */
  else if ((stats.num_white_moves == 0) && (stats.white_in_check) && (who_moves == WHITE))
    { printf("0-1 {Black Mates}\n"); fflush(stdout); return TRUE; }   /* White is mated */
  else if ((stats.num_white_moves == 0) && (!stats.white_in_check) && (who_moves == WHITE))
    { printf("1/2-1/2 {Stalemate}\n"); fflush(stdout); return TRUE; }   /* White is stalemated */
  return FALSE;
  }

/* promotes the pawn on row, column to the given piece. */
void promote_pawn (int row, int column, int topiece)
  {
    //printf("promoted to %d\n",topiece);
  pos.board[row][column].piece = topiece;
  updateChecksum(row,column);
  /* update bitboards */
  position[PAWN-1][pos.board[row][column].colour]=position[PAWN-1][pos.board[row][column].colour]&~mask[row*8+column];
  position[topiece-1][pos.board[row][column].colour]=position[topiece-1][pos.board[row][column].colour]|mask[row*8+column];
  }

/* unpromotes the piece on row, column back to a pawn. 
 * Needed for move undo's
 */
void unpromote_pawn (int row, int column)
  {
    //printf("Unpromote\n");
  position[pos.board[row][column].piece-1][pos.board[row][column].colour]=position[pos.board[row][column].piece-1][pos.board[row][column].colour]&~mask[row*8+column];
  pos.board[row][column].piece = PAWN;
  position[PAWN-1][pos.board[row][column].colour]=position[PAWN-1][pos.board[row][column].colour]|mask[row*8+column];
  updateChecksum(row,column);
  }

int checkPawn(void)
  {
  if (pos.board[1][6].piece==PAWN && pos.board[1][6].colour==WHITE)
    return TRUE;
  return FALSE;
  }


/* This procedure actually moves the pieces in the global position. It is
 * the users responibility to make sure that the given move is valid.
 * if the move results in the promotion of a pawn,... then the move structure
 * is updated appropriately. The piece-square table scores are updated as well.
 */
void makeMove (struct Move *mov)
  {
  int enpassent = FALSE;
  int colour = pos.board[mov->from_row][mov->from_column].colour;
  int piece = pos.board[mov->from_row][mov->from_column].piece;

  //printf("Making %d%d%d%d P:%d pr:%d\n",mov->from_row,mov->from_column, mov->to_row, mov->to_column,mov->piece,mov->promote); 


  /* Special case 1: castling */
  if (pos.board[mov->from_row][mov->from_column].piece == KING)
    {
    if (pos.board[mov->from_row][mov->from_column].colour == WHITE)
      {
      if ((mov->from_row == 0) && (mov->from_column == 4) && (mov->to_row == 0) && (mov->to_column == 6))
	{
	/* kingside castle for white */
        pos.board[0][5].colour = WHITE;
	pos.board[0][5].piece = ROOK;
        updateChecksum(0,5);
	pos.board[0][7].piece = EMPTY;
        updateChecksum(0,7);
        position[ROOK-1][WHITE]=position[ROOK-1][WHITE]&~mask[7];
        allpieces[WHITE]=allpieces[WHITE]&~mask[7];
        position[ROOK-1][WHITE]=position[ROOK-1][WHITE]|mask[5];
        allpieces[WHITE]=allpieces[WHITE]|mask[5];
	pos.wkRook = pos.movenum;
        pos.wcastle = pos.movenum;
        piecescores[colour]-=pcsq[WHITE][ROOK-1][0][7];
        piecescores[colour]+=pcsq[WHITE][ROOK-1][0][5];
	}
      if ((mov->from_row == 0) && (mov->from_column == 4) && (mov->to_row == 0) && (mov->to_column == 2))
	{
	/* queenside castle for white */
	pos.board[0][3].colour = WHITE;
	pos.board[0][3].piece = ROOK;
        updateChecksum(0,3);
	pos.board[0][0].piece = EMPTY;
        updateChecksum(0,0);
        position[ROOK-1][WHITE]=position[ROOK-1][WHITE]&~mask[0];
        allpieces[WHITE]=allpieces[WHITE]&~mask[0];
        position[ROOK-1][WHITE]=position[ROOK-1][WHITE]|mask[3];
        allpieces[WHITE]=allpieces[WHITE]|mask[3];
	pos.wqRook = pos.movenum;
        pos.wcastle = pos.movenum;
        piecescores[colour]-=pcsq[WHITE][ROOK-1][0][0];
        piecescores[colour]+=pcsq[WHITE][ROOK-1][0][3];
	}
      }
    if (pos.board[mov->from_row][mov->from_column].colour == BLACK)
      {
      if ((mov->from_row == 7) && (mov->from_column == 4) && (mov->to_row == 7) && (mov->to_column == 6))
	{
        /* kingside castle for black */
	pos.board[7][5].colour = BLACK;
	pos.board[7][5].piece = ROOK;
        updateChecksum(7,5);
	pos.board[7][7].piece = EMPTY;
        updateChecksum(7,7);
        position[ROOK-1][BLACK]=(position[ROOK-1][BLACK]&~mask[63])|mask[61];
        allpieces[BLACK]=(allpieces[BLACK]&~mask[63])|mask[61];  
	pos.bkRook = pos.movenum;
        pos.bcastle = pos.movenum;
        piecescores[colour]-=pcsq[BLACK][ROOK-1][7][7];
        piecescores[colour]+=pcsq[BLACK][ROOK-1][7][5];
	}
      if ((mov->from_row == 7) && (mov->from_column == 4) && (mov->to_row == 7) && (mov->to_column == 2))
	{
	/* queenside castle for black */
	pos.board[7][3].colour = BLACK;
	pos.board[7][3].piece = ROOK;
        updateChecksum(7,3);
	pos.board[7][0].piece = EMPTY;
        updateChecksum(7,0);
        position[ROOK-1][BLACK]=(position[ROOK-1][BLACK]&~mask[56])|mask[59];
        allpieces[BLACK]=(allpieces[BLACK]&~mask[56])|mask[59];
	pos.bqRook = pos.movenum;
        pos.bcastle = pos.movenum;
        piecescores[colour]-=pcsq[BLACK][ROOK-1][7][0];
        piecescores[colour]+=pcsq[BLACK][ROOK-1][7][3];
	}
      }
    }

  /* Special Case 2: en passent  (that's french for 'in passing' :) */
  if ((pos.board[mov->from_row][mov->from_column].piece == PAWN) && (pos.board[mov->to_row][mov->to_column].piece == EMPTY))
    {
    if (pos.board[mov->from_row][mov->from_column].colour == WHITE)
      if (((mov->to_row-mov->from_row)==1) && (fabs(mov->to_column-mov->from_column)==1))
	{
        /* en passent for white, kill the other pawn */
	pos.board[mov->to_row-1][mov->to_column].piece = EMPTY;
        updateChecksum(mov->to_row-1,mov->to_column);
        position[PAWN-1][!colour]=position[PAWN-1][!colour]&~mask[(mov->to_row-1)*8+mov->to_column];
        allpieces[!colour]=allpieces[!colour]&~mask[(mov->to_row-1)*8+mov->to_column];
        piecescores[!colour] -= pcsq[!colour][PAWN-1][mov->to_row-1][mov->to_column];
        enpassent = TRUE;
	}
    if (pos.board[mov->from_row][mov->from_column].colour == BLACK)
      if (((mov->to_row-mov->from_row)==-1) && (fabs(mov->to_column-mov->from_column)==1))
	{
        /* en passent for black, kill the other pawn */
	pos.board[mov->to_row+1][mov->to_column].piece = EMPTY;
        updateChecksum(mov->to_row+1,mov->to_column);
        position[PAWN-1][!colour]=position[PAWN-1][!colour]&~mask[(mov->to_row+1)*8+mov->to_column];
        allpieces[!colour]=allpieces[!colour]&~mask[(mov->to_row+1)*8+mov->to_column];
        piecescores[!colour] -= pcsq[!colour][PAWN-1][mov->to_row+1][mov->to_column];
        enpassent = TRUE;
	}
    }

  /* here we actually make the move & update the piece-square scores. */
  /* 1. Get rid of the captured piece */
  if ((mov->piece != EMPTY) && (!enpassent))
    {
    piecescores[!colour] -= pcsq[!colour][pos.board[mov->to_row][mov->to_column].piece-1][mov->to_row][mov->to_column];
    position[mov->piece-1][!colour]=position[mov->piece-1][!colour]&~mask[mov->to_row*8+mov->to_column];
    allpieces[!colour]=allpieces[!colour]&~mask[mov->to_row*8+mov->to_column];
    }  

  pos.board[mov->to_row][mov->to_column].colour = pos.board[mov->from_row][mov->from_column].colour;
  pos.board[mov->to_row][mov->to_column].piece = pos.board[mov->from_row][mov->from_column].piece;
  updateChecksum(mov->to_row,mov->to_column);
  pos.board[mov->from_row][mov->from_column].piece = EMPTY;
  updateChecksum(mov->from_row,mov->from_column);

  position[piece-1][colour]=(position[piece-1][colour]&~mask[mov->from_row*8+mov->from_column])|mask[mov->to_row*8+mov->to_column];
  allpieces[colour]=(allpieces[colour]&~mask[mov->from_row*8+mov->from_column])|mask[mov->to_row*8+mov->to_column];
  
  piecescores[colour] -= pcsq[colour][pos.board[mov->to_row][mov->to_column].piece-1][mov->from_row][mov->from_column];
  piecescores[colour] += pcsq[colour][pos.board[mov->to_row][mov->to_column].piece-1][mov->to_row][mov->to_column];
  
  /* Special Case 3: Pawn Promotion */
  if ((pos.board[mov->to_row][mov->to_column].piece == PAWN) && ((mov->to_row==0)||(mov->to_row==7)))
    {
    promote_pawn(mov->to_row,mov->to_column, mov->promote);
    piecescores[colour]-=pcsq[colour][PAWN-1][mov->to_row][mov->to_column];
    piecescores[colour]+=pcsq[colour][mov->promote-1][mov->to_row][mov->to_column];
    }

  /* UPDATE PIECE FLAGS
   * if the move was a castle, then the rook piece flag has already been
   * updated above. If a rook which hasn't moved is captured,... we set it's
   * move flag as well.
   */
  if ((mov->to_row == 7) && (mov->to_column == 7) && (pos.bkRook == 0))
    pos.bkRook = pos.movenum;
  else if ((mov->to_row == 7) && (mov->to_column == 0) && (pos.bqRook == 0))
    pos.bqRook = pos.movenum;
  else if ((mov->to_row == 0) && (mov->to_column == 0) && (pos.wqRook == 0))
    pos.wqRook = pos.movenum;
  else if ((mov->to_row == 0) && (mov->to_column == 7) && (pos.wkRook == 0))
    pos.wkRook = pos.movenum;
  if ((mov->from_row == 0) && (mov->from_column == 4) && (pos.wKing == 0))
    pos.wKing = pos.movenum;
  else if ((mov->from_row == 7) && (mov->from_column == 4) && (pos.bKing == 0))
    pos.bKing = pos.movenum;
  else if ((mov->from_row == 0) && (mov->from_column == 0) && (pos.wqRook == 0))
    pos.wqRook = pos.movenum;
  else if ((mov->from_row == 0) && (mov->from_column == 7) && (pos.wkRook == 0))
    pos.wkRook = pos.movenum;
  else if ((mov->from_row == 7) && (mov->from_column == 0) && (pos.bqRook == 0))
    pos.bqRook = pos.movenum;
  else if ((mov->from_row == 7) && (mov->from_column == 7) && (pos.bkRook == 0))
    pos.bkRook = pos.movenum;
  /* check double pawn moves */
  pos.ippawn=-1;
  if ((mov->from_row == 1) && (mov->to_row == 3) && (pos.board[mov->to_row][mov->to_column].piece == PAWN))
    {
    pos.wPawn[mov->from_column] = pos.movenum;
    pos.ippawn = mov->from_column;
    }
  if ((mov->from_row == 6) && (mov->to_row == 4) && (pos.board[mov->to_row][mov->to_column].piece == PAWN))
    {
    pos.bPawn[mov->from_column] = pos.movenum;
    pos.ippawn = mov->from_column;
    }
  /* Check if king has moved */
  if (pos.board[mov->to_row][mov->to_column].piece==KING)
    {
    if (pos.board[mov->to_row][mov->to_column].colour==WHITE)
      {
      pos.wKingRow=mov->to_row;
      pos.wKingColumn=mov->to_column;
      }
    else
      {
      pos.bKingRow=mov->to_row;
      pos.bKingColumn=mov->to_column;
      }
    }
  
  pos.movenum++;
  pos.who_moves=!pos.who_moves;

  /* now add the new position to the checksum list */
  /* note: clearing list upon capture or pawn moves would speed repetition detection process
   * However, we need to get this info back when we undo the move,.. and that's a bit of a 
   * problem!                        
   */ 
  addChecksum(&replist);
  }




/* the opposite of above. Is called with the last move made */
void unmakeMove(struct Move mov)
  {
  int i;
  int colour = pos.board[mov.to_row][mov.to_column].colour;


  //printf("Unmaking %d%d%d%d pc:%d pr:%d\n",mov.from_row, mov.from_column, mov.to_row, mov.to_column, mov.piece, mov.promote);

  pos.movenum--;                           
  /* Check the Piece Flags
   * The piece flags hold the # of the move when the piece was first moved. Used so we know
   * if castling and en passant are valid moves. 
   */
  if (pos.wKing == pos.movenum)
    pos.wKing = 0;
  else if (pos.bKing == pos.movenum)
    pos.bKing = 0;
  else if (pos.wqRook == pos.movenum)
    pos.wqRook = 0;
  else if (pos.wkRook == pos.movenum)
    pos.wkRook = 0;
  else if (pos.bqRook == pos.movenum)
    pos.bqRook = 0;
  else if (pos.bkRook == pos.movenum)
    pos.bkRook = 0;
  for (i=0;i<8;i++)
    {
    if (pos.wPawn[i] == pos.movenum)
      { pos.wPawn[i] = 0; break; }
    else if (pos.bPawn[i] == pos.movenum)
      { pos.bPawn[i] = 0; break; }
    }
  pos.ippawn=-1;
  for (i=0;i<8;i++)
    {
    if ((pos.wPawn[i]==pos.movenum-1)||(pos.bPawn[i]==pos.movenum-1))
      if (pos.movenum!=1)       /* all pawn flags default to move 0 */
        pos.ippawn=i;
    }
  /* Check if a king has moved */
  if (pos.board[mov.to_row][mov.to_column].piece==KING)
    {
    if (colour==WHITE)
      {
      pos.wKingRow=mov.from_row;
      pos.wKingColumn=mov.from_column;
      }
    else
      {
      pos.bKingRow=mov.from_row;
      pos.bKingColumn=mov.from_column;
      }
    }

  /* Check the 4 castling special cases and move the rook! */
  if ((mov.from_row == 0) && (mov.from_column == 4) && (pos.board[mov.to_row][mov.to_column].piece == KING))
    {
    if ((mov.to_row == 0) && (mov.to_column == 6))
      {
      /* White Kingside */
      pos.board[0][7].piece = ROOK;
      pos.board[0][7].colour = WHITE;
      updateChecksum(0,7);
      pos.board[0][5].piece = EMPTY;
      updateChecksum(0,5);
      position[ROOK-1][colour]=(position[ROOK-1][colour]&~mask[5])|mask[7];
      allpieces[colour]=(allpieces[colour]&~mask[5])|mask[7];
      pos.wkRook = 0;
      pos.wcastle = FALSE;
      piecescores[WHITE] -= pcsq[WHITE][ROOK-1][0][5];
      piecescores[WHITE] += pcsq[WHITE][ROOK-1][0][7];
      }
    if ((mov.to_row == 0) && (mov.to_column == 2))
      {
      /* White Queenside */
      pos.board[0][0].piece = ROOK;
      pos.board[0][0].colour = WHITE;
      updateChecksum(0,0);
      pos.board[0][3].piece = EMPTY;
      updateChecksum(0,3);
      position[ROOK-1][colour]=(position[ROOK-1][colour]&~mask[3])|mask[0];
      allpieces[colour]=(allpieces[colour]&~mask[3])|mask[0];
      pos.wqRook = 0;
      pos.wcastle = FALSE;
      piecescores[WHITE] -= pcsq[WHITE][ROOK-1][0][3];
      piecescores[WHITE] += pcsq[WHITE][ROOK-1][0][0];
      }
    }
  if ((mov.from_row == 7) && (mov.from_column == 4) && (pos.board[mov.to_row][mov.to_column].piece == KING))
    {
    if ((mov.to_row == 7) && (mov.to_column == 6))
      {
      /* Black Kingside */
      pos.board[7][7].piece = ROOK;
      pos.board[7][7].colour = BLACK;
      updateChecksum(7,7);
      pos.board[7][5].piece = EMPTY;
      updateChecksum(7,5);
      position[ROOK-1][colour]=(position[ROOK-1][colour]&~mask[61])|mask[63];
      allpieces[colour]=(allpieces[colour]&~mask[61])|mask[63];
      pos.bkRook = 0;
      pos.bcastle = FALSE;
      piecescores[BLACK] -= pcsq[BLACK][ROOK-1][7][5];
      piecescores[BLACK] += pcsq[BLACK][ROOK-1][7][7];
      }
    if ((mov.to_row == 7) && (mov.to_column == 2))
      {
      /* Black Queenside */
      pos.board[7][0].piece = ROOK;
      pos.board[7][0].colour = BLACK;
      updateChecksum(7,0);
      pos.board[7][3].piece = EMPTY;
      updateChecksum(7,3);
      position[ROOK-1][colour]=(position[ROOK-1][colour]&~mask[59])|mask[56];
      allpieces[colour]=(allpieces[colour]&~mask[59])|mask[56];
      pos.bqRook = 0;
      pos.bcastle = FALSE;
      piecescores[BLACK] -= pcsq[BLACK][ROOK-1][7][3];
      piecescores[BLACK] += pcsq[BLACK][ROOK-1][7][0];
      }
    }
  
  /* Check for a promoted pawn and depromote it */
  if (mov.promote != EMPTY)
    {
    unpromote_pawn(mov.to_row, mov.to_column);
    piecescores[colour] -= pcsq[colour][mov.promote-1][mov.to_row][mov.to_column];
    piecescores[colour] += pcsq[colour][PAWN-1][mov.to_row][mov.to_column];
    }
  
  /* Now we do the normal undo case, move the piece on the to square back
   * to the from square and replace the from square with the captured piece
   */
  /* 1/2 return the moved piece back to it's original square */
  position[pos.board[mov.to_row][mov.to_column].piece-1][colour]=(position[pos.board[mov.to_row][mov.to_column].piece-1][colour]&~mask[mov.to_row*8+mov.to_column])|mask[mov.from_row*8+mov.from_column];
  allpieces[colour]=(allpieces[colour]&~mask[mov.to_row*8+mov.to_column])|mask[mov.from_row*8+mov.from_column];
  pos.board[mov.from_row][mov.from_column].piece = pos.board[mov.to_row][mov.to_column].piece;
  pos.board[mov.from_row][mov.from_column].colour = pos.board[mov.to_row][mov.to_column].colour;
  updateChecksum(mov.from_row,mov.from_column);
  piecescores[colour]-=pcsq[colour][pos.board[mov.from_row][mov.from_column].piece-1][mov.to_row][mov.to_column];
  piecescores[colour]+=pcsq[colour][pos.board[mov.from_row][mov.from_column].piece-1][mov.from_row][mov.from_column];
 
  /* 2/2 return the captured piece to it's square */
  if (mov.piece != ENPASSENTPAWN)
    {
    if (mov.piece)
      {
      position[mov.piece-1][!colour]=position[mov.piece-1][!colour]|mask[mov.to_row*8+mov.to_column];
      allpieces[!colour]=allpieces[!colour]|mask[mov.to_row*8+mov.to_column];
      }
    pos.board[mov.to_row][mov.to_column].piece = mov.piece;
    pos.board[mov.to_row][mov.to_column].colour = !pos.board[mov.to_row][mov.to_column].colour;
    updateChecksum(mov.to_row,mov.to_column);
    if (mov.piece != EMPTY)
      piecescores[!colour]+=pcsq[!colour][mov.piece-1][mov.to_row][mov.to_column];
    }
  /* the en passant special case is handled here */
  else
    {
    pos.board[mov.to_row][mov.to_column].piece = EMPTY;
    if (pos.board[mov.from_row][mov.from_column].colour == WHITE)
      {
      pos.board[mov.to_row-1][mov.to_column].piece = PAWN;
      pos.board[mov.to_row-1][mov.to_column].colour = BLACK;
      updateChecksum(mov.to_row-1,mov.to_column);
      position[PAWN-1][BLACK]=position[PAWN-1][BLACK]|mask[(mov.to_row-1)*8+mov.to_column];
      allpieces[BLACK]=allpieces[BLACK]|mask[(mov.to_row-1)*8+mov.to_column];
      piecescores[!colour]+=pcsq[!colour][PAWN-1][mov.to_row-1][mov.to_column];
      }
    else
      {
      pos.board[mov.to_row+1][mov.to_column].piece = PAWN;
      pos.board[mov.to_row+1][mov.to_column].colour = WHITE;
      updateChecksum(mov.to_row+1, mov.to_column);
      position[PAWN-1][WHITE]=position[PAWN-1][WHITE]|mask[(mov.to_row+1)*8+mov.to_column];
      allpieces[WHITE]=allpieces[WHITE]|mask[(mov.to_row+1)*8+mov.to_column];
      piecescores[!colour]+=pcsq[!colour][PAWN-1][mov.to_row+1][mov.to_column];
      }
    }

  pos.who_moves=!pos.who_moves;
  /* remove last move from the checksum list */
  deleteChecksum(&replist);
  }

















