/* This file contains routines for the input/output of FEN positional notation. */


/* Opens the given file and returns the number'th ordinal FEN 
   returns FALSE if the file couldn't be opened or if there are
   no more FEN's in the file.
 */
int getFEN(char *FileName, int number, char *fen)
  {
  int i;
  FILE *fp;

  if((fp = fopen(FileName,"rt"))==NULL)
    return FALSE;

  for (i=0;i<(number*2)-1;i++)
    if(fgets(fen, 200, fp)==NULL) return FALSE;
    
  removeNewline(fen);

  if (strlen(fen)<5) return FALSE;

  fclose(fp);
  return TRUE;
  }

int getSolution(char *FileName, int number, char *solution)
  {
  int i;
  FILE *fp;
  char temp[200];
  
  if ((fp = fopen(FileName,"rt"))==NULL)
    return FALSE;
 
  for (i=0;i<(number-1)*2+1;i++)
    if (fgets(temp,180, fp)==NULL) return FALSE;
    
  fscanf(fp, "%s", solution);
  
  fclose(fp);
  return TRUE;
  }

int setupFENrow(char *s, int row)
  {
  int i=0;
  int column;

  for (column=0;column<8;column++)
    {
    if (s[i]=='K') {pos.board[row][column].piece=KING; pos.board[row][column].colour=WHITE;}
    else if (s[i]=='k') {pos.board[row][column].piece=KING; pos.board[row][column].colour=BLACK;}
    else if (s[i]=='Q') {pos.board[row][column].piece=QUEEN; pos.board[row][column].colour=WHITE;}     
    else if (s[i]=='q') {pos.board[row][column].piece=QUEEN; pos.board[row][column].colour=BLACK;}     
    else if (s[i]=='R') {pos.board[row][column].piece=ROOK; pos.board[row][column].colour=WHITE;}     
    else if (s[i]=='r') {pos.board[row][column].piece=ROOK; pos.board[row][column].colour=BLACK;}     
    else if (s[i]=='N') {pos.board[row][column].piece=KNIGHT; pos.board[row][column].colour=WHITE;}     
    else if (s[i]=='n') {pos.board[row][column].piece=KNIGHT; pos.board[row][column].colour=BLACK;}     
    else if (s[i]=='B') {pos.board[row][column].piece=BISHOP; pos.board[row][column].colour=WHITE;}     
    else if (s[i]=='b') {pos.board[row][column].piece=BISHOP; pos.board[row][column].colour=BLACK;}     
    else if (s[i]=='P') {pos.board[row][column].piece=PAWN; pos.board[row][column].colour=WHITE;}     
    else if (s[i]=='p') {pos.board[row][column].piece=PAWN; pos.board[row][column].colour=BLACK;}
    else if (isdigit(s[i])) column+=atoi(&s[i])-1;
    else return FALSE;   /* Invalid Character in INPUT */
    i++;
    }
  return TRUE;
  }

int parseCastlePermissions(char *s)
  {
  int i;
  int wk,wq,bk,bq;

  wk=wq=bk=bq=FALSE;

  pos.wkRook=-1;
  pos.wqRook=-1;
  pos.bkRook=-1;
  pos.bqRook=-1;

  if (s[0]=='-')   /* no castling allowed */
    return TRUE;
   
  for (i=0;i<strlen(s);i++)
    {     
    if (s[i]=='K') wk=TRUE;
    else if (s[i]=='k') bk=TRUE;
    else if (s[i]=='Q') wq=TRUE;
    else if (s[i]=='q') bq=TRUE;
    else return FALSE;
    }

  /* Now we know which permissions are set,... we need to setup the internal variables accordingly. */
  if (wk) {pos.wkRook=0;pos.wKing=0;}
  if (wq) {pos.wqRook=0;pos.wKing=0;}
  if (bk) {pos.bkRook=0;pos.bKing=0;}
  if (bq) {pos.bqRook=0;pos.bKing=0;}
  return TRUE;
  }

int parseEnpassantAvailability(char *s)
  {
  if (s[0]=='-') return TRUE;
  else
    {
    if (s[0]=='a') pos.ippawn=0;
    else if (s[0]=='b') pos.ippawn=1;
    else if (s[0]=='c') pos.ippawn=2;
    else if (s[0]=='d') pos.ippawn=3;
    else if (s[0]=='e') pos.ippawn=4;
    else if (s[0]=='f') pos.ippawn=5;
    else if (s[0]=='g') pos.ippawn=6;
    else if (s[0]=='h') pos.ippawn=7;
    else return FALSE;
    }
  return TRUE;
  } 

/* this initializes the bitboards,.. piecesscores,.. ect... */
void initFENOther(void)
  {
  int i, j;

  setupBitboards();
  initPiecescores();

  /* find the kings */
  for (i=0;i<8;i++)
    for (j=0;j<8;j++)
      {
      if (pos.board[i][j].piece==KING)
	{
        if (pos.board[i][j].colour==WHITE)
	  {
          pos.wKingRow=i;
          pos.wKingColumn=j;
	  }
        else
          {
          pos.bKingRow=i;
          pos.bKingColumn=j;
	  }
	}
      }
  }

/* This will parse a FEN string and set up the board positions */
int setupFEN(char *fen)
  {
  char current[200];
  char *temp;
  char stuff[100];
  int i;

  strcpy(current, fen);
  clearBoard();      /* empty the position */

  /* First, parse the positional stuff */
  for (i=7;i>=0;i--)
    {
    temp=strchr(current,'/');
    if (temp==NULL) temp=strchr(current,' ');  /* the last one ends with a space */
    strcpy(stuff,"");
    strncat(stuff, current, (temp-current)/sizeof(char));
    strcpy(current, temp+1);
    if(!setupFENrow(stuff, i)) printf("ERROR SETTING UP ROW\n");
    }
   
  /* now we parse the rest of the fields */
  for (i=0;i<4;i++)
    {
    temp=strchr(current,' ');
    if (temp==NULL) return FALSE;
    strcpy(stuff,"");
    strncat(stuff, current, (temp-current)/sizeof(char));
    strcpy(current, temp+1);
    if (i==0)
      {
	/* active colour field */
      if (stuff[0]=='w') pos.who_moves=WHITE;
      else if (stuff[0]=='b') pos.who_moves=BLACK;
      else {printf("Invalid Field 2 of FEN position\n");return FALSE;}
      }
    if (i==1)
      {
	/* castle permissions field */
      parseCastlePermissions(stuff);
      }
    if (i==2)
      {
	/* enpasssant availability */
      parseEnpassantAvailability(stuff);
      }
    if (i==3)
      {
      
	/* stuff contains halfmove clock - I don't support this yet! (50 move rule) */
      }
    }

  /* The last field is the move # (full move #) */
  pos.movenum = atoi(current)*2;
  if (pos.who_moves==WHITE) pos.movenum--;

  /* Lastly,.. we need to setup the remaining stuff... */
  initFENOther();
  collectStats();
  return TRUE;  
  }

/* Performs a test on a file of FEN positions.
 * Searches every position until the set time is elapsed.
 * Outputs analysis results to screen and to file */
void FENTest(char *FileName)
  {
  char fen[200];
  char solution[15];
  int i=1;
  double posEdge;
  struct MoveList history;
  struct Move best;
  struct Move solutionMove;
  int numRight=0, numWrong=0;


  if ((fentestfile=fopen("testout.txt","wt"))==NULL)
    printf("ERROR OPENING TEST OUTPUT FILE\n");

  fprintf(fentestfile, "FEN Test Results Summary for input file [%s].\n",FileName);
  fprintf(fentestfile, "Positional Chess Analysis by %s.\n\n",ProgramName);

  initList(&history);
  analysis=TRUE;

  while(getFEN(FileName, i, fen))
    {
    fprintf(fentestfile, "Position #%d: %s\n",i,fen);
    setupFEN(fen);
    posEdge=(double)(piecescores[WHITE]-piecescores[BLACK])/100;
    if (posEdge>=0)
      fprintf(fentestfile, "PosEdge: +%.2f P [White]\n",posEdge);
    else
      fprintf(fentestfile, "PosEdge: %.2f P [White]\n",posEdge);
    getSolution(FileName, i, solution);
    moveCopy(parseAlgebra(solution),&solutionMove);
    createAlgebraOutput(solutionMove, solution);
    fprintf(fentestfile, "Solution:  %s\n",solution);
    drawBoard();
    printf("Position #%d: %s\n",i,fen);
    printf("Solution:  %s\n",solution);
    moveCopy(think(history),&best);
    if (moveCompare(best, solutionMove))
      {
      printf("Correct!\n"); 
      fprintf(fentestfile, "Correct!\n");
      numRight++;
      }
    else
      {
      printf("Wrong!\n");
      fprintf(fentestfile, "Wrong!\n");
      numWrong++;
      }
    printf("\n");
    fprintf(fentestfile, "\n");
    i++;
    }

  printf("\n\n"); fprintf(fentestfile,"\n\n");
  printf("Score: %d/%d = %d\%\n", numRight, numRight+numWrong,(int)(((float)numRight/(float)(numRight+numWrong))*100));
  fprintf(fentestfile, "Score: %d/%d = %d\%\n", numRight, numRight+numWrong,(int)(((float)numRight/(float)(numRight+numWrong))*100));
  printf("Time/Pos: %ldms\n",TIME);
  fprintf(fentestfile, "Time/Pos: %ldms\n", TIME);  

  fclose(fentestfile);
  }


void handleFEN(char *FileName)
  {
  char fen[200];
  
  if (!getFEN(FileName, 1, fen))
    {
    printf("Input Error");
    return;
    }

  else
    {
    printf("First FEN is: %s\n",fen);
    setupFEN(fen);
    return;
    }
  }












