/* Functions to generate move lists */


/* fills up the movelist with all the moves the given colour can make from
 * the current position.
 * Moves that result in capture go on the front of the list and moves that
 * don't get appended to the end. This speeds up the alpha-beta search as
 * it results in faster pruning.
 * Modes are 0: normal (all moves) 1: only capture moves 2: only nonCapture 
 * moves.
 */
void generateMoves(struct MoveList *ml, int mode)
  {
  int i,j,k,l;
  clock_t genStartTime, genEndTime;
  struct Move tempmove;

  genStartTime=getMS();

  if (ml->size > 0)
    clearList(ml);
  initList(ml);

  for (i=0;i<8;i++)
    {
    for (j=0;j<8;j++)
      {
      /* find all colour's pieces */
      if ((pos.board[i][j].colour == pos.who_moves) && (pos.board[i][j].piece > 0))
	{
	tempmove.from_row = i;
	tempmove.from_column = j;
	if (pos.board[i][j].piece == PAWN)
	  {
	  /* check move 1 or 2 forward and 2 killing squares */
	  if (pos.board[i][j].colour == WHITE)
	    {
	    if ((mode==0)||(mode==2))
	      {
              tempmove.to_row = i+1;       /* 1 forward */
	      tempmove.to_column = j;
	      tempmove.piece = EMPTY;
	      if (tempmove.to_row < 7)
                {
                tempmove.promote = EMPTY;
                if (good_move(tempmove))
                  appendMove(tempmove, ml);
                }
              else
                {
                tempmove.promote = QUEEN;
                if (good_move(tempmove))
                  {      /* we add because promotions are good :) */
                  tempmove.promote = QUEEN; addMove(tempmove, ml);
                  tempmove.promote = ROOK; addMove(tempmove, ml);
                  tempmove.promote = KNIGHT; addMove(tempmove, ml);
                  tempmove.promote = BISHOP; addMove(tempmove, ml);
                  }
	        }
	      tempmove.to_row = i+2;       /* 2 forward */
	      tempmove.to_column = j;
	      tempmove.piece = EMPTY;
              tempmove.promote = EMPTY;
	      if (good_move(tempmove))
	        appendMove(tempmove, ml);
	      }
	    if ((mode==1)||(mode==0))
              {
              tempmove.to_row = i+1;       /* kill to left */
	      tempmove.to_column = j-1;
	      tempmove.piece = pos.board[i+1][j-1].piece;
              if (tempmove.to_row < 7)
                {
                if (tempmove.piece == EMPTY) tempmove.piece = ENPASSENTPAWN;
                tempmove.promote = EMPTY;
                if (good_move(tempmove))
                  addMove(tempmove, ml);
	        }
              else           /* kill & promotion */
                {
                tempmove.promote = QUEEN;
                if (good_move(tempmove))
                  {
                  tempmove.promote = QUEEN; addMove(tempmove, ml);
                  tempmove.promote = ROOK; addMove(tempmove, ml);
                  tempmove.promote = KNIGHT; addMove(tempmove, ml);
                  tempmove.promote = BISHOP; addMove(tempmove, ml);
		  }
	        }
	      tempmove.to_row = i+1;       /* kill to right */
	      tempmove.to_column = j+1;
	      tempmove.piece = pos.board[i+1][j+1].piece;
	      if (tempmove.to_row < 7)
                {
                if (tempmove.piece == EMPTY) tempmove.piece = ENPASSENTPAWN;
                tempmove.promote = EMPTY;
                if (good_move(tempmove))
                  addMove(tempmove, ml);
	        }
              else           /* kill & promotion */
                {
                tempmove.promote = QUEEN;
                if (good_move(tempmove))
                  {
                  tempmove.promote = QUEEN; addMove(tempmove, ml);
                  tempmove.promote = ROOK; addMove(tempmove, ml);
                  tempmove.promote = KNIGHT; addMove(tempmove, ml);
                  tempmove.promote = BISHOP; addMove(tempmove, ml);
	 	  }
	        }   
	      }
	    }
	  if (pos.board[i][j].colour == BLACK)
	    {
            if ((mode==0)||(mode==2))
              {
	      tempmove.to_row = i-1;       /* 1 forward */
	      tempmove.to_column = j;
	      tempmove.piece = EMPTY;
              if (tempmove.to_row > 0)
                {
                tempmove.promote = EMPTY;
                if (good_move(tempmove))
                  appendMove(tempmove, ml);
	        }
              else                /* promotion */
                {
                tempmove.promote = QUEEN;
                if (good_move(tempmove))
                  {
                  tempmove.promote = QUEEN; addMove(tempmove, ml);
                  tempmove.promote = ROOK; addMove(tempmove, ml);
                  tempmove.promote = BISHOP; addMove(tempmove, ml);
                  tempmove.promote = KNIGHT; addMove(tempmove, ml);
		  }
	        }
	      tempmove.to_row = i-2;       /* 2 forward */
	      tempmove.to_column = j;
	      tempmove.piece = EMPTY;
              tempmove.promote = EMPTY;
	      if (good_move(tempmove))
	        appendMove(tempmove, ml);
	      }
            if ((mode==1)||(mode==0))
              {
	      tempmove.to_row = i-1;       /* kill to left */
	      tempmove.to_column = j-1;
	      tempmove.piece = pos.board[i-1][j-1].piece;
              if (tempmove.to_row > 0)
                {
                if (tempmove.piece == EMPTY) tempmove.piece = ENPASSENTPAWN;
                tempmove.promote = EMPTY;
                if (good_move(tempmove))
                  appendMove(tempmove, ml);
	        }
              else                /* promotion */
                {
                tempmove.promote = QUEEN;
                if (good_move(tempmove))
                  {
                  tempmove.promote = QUEEN; addMove(tempmove, ml);
                  tempmove.promote = ROOK; addMove(tempmove, ml);
                  tempmove.promote = BISHOP; addMove(tempmove, ml);
                  tempmove.promote = KNIGHT; addMove(tempmove, ml);
		  }
	        }
	      tempmove.to_row = i-1;       /* kill to right */
	      tempmove.to_column = j+1;
	      tempmove.piece = pos.board[i-1][j+1].piece;
              tempmove.promote = EMPTY;
              if (tempmove.to_row > 0)
                {
                if (tempmove.piece == EMPTY) tempmove.piece = ENPASSENTPAWN;
                tempmove.promote = EMPTY;
                if (good_move(tempmove))
                  appendMove(tempmove, ml);
	        }
              else                /* promotion */
                {
                tempmove.promote = QUEEN;
                if (good_move(tempmove))
                  {
                  tempmove.promote = QUEEN; addMove(tempmove, ml);
                  tempmove.promote = ROOK; addMove(tempmove, ml);
                  tempmove.promote = BISHOP; addMove(tempmove, ml);
                  tempmove.promote = KNIGHT; addMove(tempmove, ml);
		  }
	        }
	      }
	    }
	  }
	if (pos.board[i][j].piece == ROOK)
	  {
	  /* check all verticle and all horizontal possibilities */
	  tempmove.to_row = i;
	  for (k=0;k<8;k++)   /* check horizontal (cycle columns) */
	    {
	    tempmove.to_column = k;
	    tempmove.piece = pos.board[i][k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
              {
	      if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  tempmove.to_column = j;
	  for (k=0;k<8;k++)    /* check verticle (cycle rows) */
	    {
	    tempmove.to_row = k;
	    tempmove.piece = pos.board[k][j].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
              {
	      if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  }
	if (pos.board[i][j].piece == BISHOP)
	  {
	  /* check all diagonal possibilities
	   * 1. up and to the right
	   */
	  l = 7 - j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i+k][j+k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
              {
	      if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
	  	  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  /* 2. up and too the left */
	  l = j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i+k][j-k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
              {
	      if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
	  	  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  /* 3. Down and to the right */
	  l = i;
	  if (7 - j < l)
	    l = 7 - j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i-k][j+k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
              {
	      if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
	  	  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
 	    }
	  /* 4. Down and to the left */
	  l = i;
	  if (j < i)
	    l = j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i-k][j-k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  }
	if (pos.board[i][j].piece == QUEEN)
	  {
	  /* check both rook & bishop possibilities
	   * check all verticle and all horizontal possibilities
	   */
	  tempmove.to_row = i;
	  for (k=0;k<8;k++)   /* 1. check horizontal (cycle columns) */
	    {
	    tempmove.to_column = k;
	    tempmove.piece = pos.board[i][k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  tempmove.to_column = j;
	  for (k=0;k<8;k++)    /* check verticle (cycle rows) */
	    {
	    tempmove.to_row = k;
	    tempmove.piece = pos.board[k][j].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  /* check all diagonal possibilities
	   * 1. up and to the right
	   */
	  l = 7 - j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i+k][j+k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  /* 2. up and too the left */
	  l = j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i+k][j-k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  /* 3. Down and to the right */
	  l = i;
	  if (7 - j < l)
	    l = 7 - j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i-k][j+k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  /* 4. Down and to the left */
	  l = i;
	  if (j < i)
	    l = j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i-k][j-k].piece;
            tempmove.promote = EMPTY;
            if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	      {
              if (good_move(tempmove))
	        if (tempmove.piece != EMPTY)
		  addMove(tempmove, ml);
	        else
		  appendMove(tempmove, ml);
	      }
	    }
	  }
	if (pos.board[i][j].piece == KNIGHT)
	  {
	  /* there are 8 possible moves */
	  tempmove.to_row = i+2;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i+2][j+1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i+2;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i+2][j-1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i+1;
	  tempmove.to_column = j+2;
	  tempmove.piece = pos.board[i+1][j+2].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i+1;
	  tempmove.to_column = j-2;
	  tempmove.piece = pos.board[i+1][j-2].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-2;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i-2][j+1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-2;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i-2][j-1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-1;
	  tempmove.to_column = j+2;
	  tempmove.piece = pos.board[i-1][j+2].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-1;
	  tempmove.to_column = j-2;
	  tempmove.piece = pos.board[i-1][j-2].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  }
	if (pos.board[i][j].piece == KING)
	  {
	  /* 10 possible moves, including the 2 castle moves */
	  tempmove.to_row = i+1;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i+1][j-1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i+1;
	  tempmove.to_column = j;
	  tempmove.piece = pos.board[i+1][j].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i+1;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i+1][j+1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i][j-1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i][j+1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-1;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i-1][j-1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-1;
	  tempmove.to_column = j;
	  tempmove.piece = pos.board[i-1][j].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY))) 
            {
	    if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i-1;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i-1][j+1].piece;
          tempmove.promote = EMPTY;
          if ((mode==0)||((mode==1)&&(tempmove.piece!=EMPTY))||((mode==2)&&(tempmove.piece==EMPTY)))
	    {
            if (good_move(tempmove))
	      if (tempmove.piece != EMPTY)
	        addMove(tempmove, ml);
	      else
	        appendMove(tempmove, ml);
	    }
	  tempmove.to_row = i;
	  tempmove.to_column = j+2;   /* kingside castle */
	  tempmove.piece = EMPTY;
          tempmove.promote = EMPTY;
          if ((mode==0) || (mode==2))
	    if (good_move(tempmove))
	      appendMove(tempmove, ml);
	  tempmove.to_row = i;
	  tempmove.to_column = j-2;   /* kingside castle */
	  tempmove.piece = EMPTY;
          tempmove.promote = EMPTY;
          if ((mode==0)||(mode==2))
	    if (good_move(tempmove))
	      appendMove(tempmove, ml);
	  }
	}
      }
    }

  genEndTime=getMS();
  timeGenerating+=(double)(genEndTime-genStartTime)/(double)1000;
  }


