/* Miscellaneous Input/Output Routines   */


void outTime(long int ms)
  {
  long int hours,minutes,seconds;
  long int temp= ms;

  hours = temp / 3600000;
  temp -= hours*3600000;
  minutes = temp / 60000;
  temp -= minutes*60000;
  seconds = temp / 1000;
  temp -= seconds * 1000;
  temp /= 10;

  printf("%02ld:%02ld:%02ld.%02ld  %10ld",hours,minutes,seconds,temp,nummoves);
  if (FENTEST)
    fprintf(fentestfile, "%02ld:%02ld:%02ld.%02ld  %10ld",hours,minutes,seconds,temp,nummoves);
  }

void removeNewline(char *string)
  {
  int i,j = 0;    // Counters
  char temp[100];  // Hold the new string

  for (i=0; i < strlen(string) + 1 ; i++)
    {
    if (string[i] != 10)
      {
      temp[j] = string[i];
      j++;
      }
    }

  strcpy(string, temp);
  }

/* takes the game result string and removes the part between
 * and including the {}'s. Used for pgn logging */
void parseResult(char *string)
  {
  int start;
  char temp[200];
  
  start = strcspn(string, "{");

  strcpy(temp,"");
  strncat(temp,string,start);
  strcpy(string, temp);
  }  

/* like atoi() but only with single characters */
int toNumber(char digit)
  {
  return -48+digit;
  }

/* This function takes xboard coordinate input (eg e2e4) and converts it to
 * the program's internal move format (eg 1 4 3 4). It is the user's
 * responsibility to ensure that the input matches the form above. If the
 * move is not a valid one (parameters are not inbounds) then the function
 * will return a move with from_row = -1
 */
struct Move parseMoveInput (char *input)
  {
  struct Move mov;
  char a,b,c,d;

  /* Initialize the Move */
  mov.from_row = 1;
  mov.from_column = 1;
  mov.to_row = 1;
  mov.to_column = 1;
  mov.piece = EMPTY;
  mov.promote = EMPTY;

  if (strlen(input)<4 || strlen(input)>5)
    {
    mov.from_row = -1;
    return mov;
    }

  /* we are only concerned with the first 5 characters of the input.
   * the first 4 are the move and the 5th (if it exists) is the 
   * promotion indicator
   */
  a = input[0];
  b = input[1];
  c = input[2];
  d = input[3];

  if (a == 'a') mov.from_column = 0;
  else if (a == 'b') mov.from_column = 1;
  else if (a == 'c') mov.from_column = 2;
  else if (a == 'd') mov.from_column = 3;
  else if (a == 'e') mov.from_column = 4;
  else if (a == 'f') mov.from_column = 5;
  else if (a == 'g') mov.from_column = 6;
  else if (a == 'h') mov.from_column = 7;
  else { mov.from_row=-1; return mov;}


  if (c == 'a') mov.to_column = 0;
  else if (c == 'b') mov.to_column = 1;
  else if (c == 'c') mov.to_column = 2;
  else  if (c == 'd') mov.to_column = 3;
  else if (c == 'e') mov.to_column = 4;
  else if (c == 'f') mov.to_column = 5;
  else if (c == 'g') mov.to_column = 6;
  else if (c == 'h') mov.to_column = 7;
  else {mov.from_row=-1; return mov;}

  mov.from_row = toNumber(b)-1;
  mov.to_row = toNumber(d)-1;

  if ((mov.to_row > 7) || (mov.to_row < 0) || (mov.to_column > 7) || (mov.to_column < 0))
    mov.from_row = -1;
  if ((mov.from_row > 7) || (mov.from_row < 0) || (mov.from_column > 7) || (mov.from_column < 0))
    mov.from_row = -1;

  if (strlen(input) > 4)
    {
    if ((input[4] == 'q') || (input[4] == 'Q'))
      mov.promote = QUEEN; 
    else if ((input[4] == 'r') || (input[4] == 'R'))
      mov.promote = ROOK; 
    else if ((input[4] == 'n') || (input[4] == 'N'))
      mov.promote = KNIGHT; 
    else if ((input[4] == 'b') || (input[4] == 'B'))
      mov.promote = BISHOP;
    } 
  
  /* We must take care of the case where a pawn is being promoted but
   * the promotion type was not indicated in the input string
   */
  if ((pos.board[mov.from_row][mov.from_column].piece==PAWN)&&((mov.to_row==7)||(mov.to_row==0)) && (mov.promote==EMPTY))
    mov.promote = QUEEN;       /* default promotion is to a queen */

  return mov;
  }

/* This function does the opposite of the one above. It takes in a move in
 * internal move format (eg 1 4 3 4) and converts it to xboard style
 * coordinate notation (eg e2e4).
 */
void createMoveOutput (struct Move mov, char *move)
  {
    if (mov.promote == EMPTY) strcpy(move, "four");
    else strcpy(move, "five5");

    //printf("CreateMoveOutput recieved [%d %d %d %d] p: %d pr: %d\n",mov.from_row,mov.from_column,mov.to_row,mov.to_column,mov.piece,mov.promote);

  if (mov.from_column == 0)  move[0] = 'a';
  else if (mov.from_column == 1)  move[0] = 'b';
  else if (mov.from_column == 2)  move[0] = 'c';
  else if (mov.from_column == 3)  move[0] = 'd';
  else if (mov.from_column == 4)  move[0] = 'e';
  else if (mov.from_column == 5)  move[0] = 'f';
  else if (mov.from_column == 6)  move[0] = 'g';
  else if (mov.from_column == 7)  move[0] = 'h';

  if (mov.to_column == 0)  move[2] = 'a';
  else if (mov.to_column == 1)  move[2] = 'b';
  else if (mov.to_column == 2)  move[2] = 'c';
  else if (mov.to_column == 3)  move[2] = 'd';
  else if (mov.to_column == 4)  move[2] = 'e';
  else if (mov.to_column == 5)  move[2] = 'f';
  else if (mov.to_column == 6)  move[2] = 'g';
  else if (mov.to_column == 7)  move[2] = 'h';

  if (mov.from_row == 0) move[1] = '1';
  else if (mov.from_row == 1) move[1] = '2';
  else if (mov.from_row == 2) move[1] = '3';
  else if (mov.from_row == 3) move[1] = '4';
  else if (mov.from_row == 4) move[1] = '5';
  else if (mov.from_row == 5) move[1] = '6';
  else if (mov.from_row == 6) move[1] = '7';
  else if (mov.from_row == 7) move[1] = '8';

  if (mov.to_row == 0) move[3] = '1';
  else if (mov.to_row == 1) move[3] = '2';
  else if (mov.to_row == 2) move[3] = '3';
  else if (mov.to_row == 3) move[3] = '4';
  else if (mov.to_row == 4) move[3] = '5';
  else if (mov.to_row == 5) move[3] = '6';
  else if (mov.to_row == 6) move[3] = '7';
  else if (mov.to_row == 7) move[3] = '8';

  if (mov.promote == QUEEN) move[4] = 'q';
  else if (mov.promote == ROOK) move[4] = 'r';
  else if (mov.promote == KNIGHT) move[4] = 'n';
  else if (mov.promote == BISHOP) move[4] = 'b';
  }

/* Removes the characters # x + ! from the input string
 * These characters are part of Algebraic Notation but
 * are not required to parse it.
 */
void remChars(char *input)
  {
  int i =0;
  char temp[30];

  while (input[i])
    {
    if ((input[i]=='#')||(input[i]=='+')||(input[i]=='x')||(input[i]=='!'))
      {
      /* remove the character from the string */
      strcpy(temp,"");
      strncat(temp,input,i);     
      strncat(temp,&(input[i+1]),strlen(input)-(i+1)); 
      strcpy(input,temp);    
      }
    else i++;
    }
  }

/* Takes an input string containing a move in Algebraic Notation and 
 * outputs a move in the internal format. A move with from_row set to
 * -2 or -1 is returned if the move is invalid (either an ambiguous move
 * or a move which doesn't exist). 
 */
struct Move parseAlgebra (char *move)
  {
  struct MoveList moves;
  struct Move mov;
  int tarcol;
  char input[10];

  strcpy(input, move);       /* So we don't disturb move */

  /* Initialize the Move */
  mov.from_row = -1;
  mov.from_column = 1;
  mov.to_row = 1;
  mov.to_column = 1;
  mov.piece = EMPTY;
  mov.promote = EMPTY;

  /* The special castling cases must be checked first because they
   * do not agree with standard algebraic notation.
   */
  if ((!strcmp(input, "o-o"))||(!strcmp(input, "0-0"))||(!strcmp(input, "O-O")))
    {
      /* kingside castle */
      if (pos.who_moves==WHITE)
        {
        mov.from_row=0;
        mov.to_row=0;
        mov.from_column=4;
        mov.to_column=6;
        }
      else
        {
        mov.from_row=7;
        mov.to_row=7;
        mov.from_column=4;
        mov.to_column=6;
        }
      return mov;
    }
  else if ((!strcmp(input,"o-o-o"))||(!strcmp(input,"0-0-0"))||(!strcmp(input,"O-O-O")))
    {
      /* queenside castle */
      if (pos.who_moves==WHITE)
        {
        mov.from_row=0;
        mov.to_row=0;
        mov.from_column=4;
        mov.to_column=2;
        }
      else
        {
        mov.from_row=7;
        mov.to_row=7;
        mov.from_column=4;
        mov.to_column=2;
        }
      return mov;
    }

  initList(&moves);
  generateMoves(&moves, 0);

  if (moves.size == 0)
    {
    mov.from_row=-1;
    return mov;
    }

  /* We now have a list of all the possible moves. Next we parse the string
   * and delete all the moves which do not agree with it. Hopefully when we
   * are done we will only have one move. If there is more than one, then the
   * move is ambiguous. If there is less than one then the move is invalid.
   */
  remChars(input);
  
  /* 1. Check the piece indicator */
  if (isupper(input[0]))
    {
    if (input[0]=='K')
      searchList(&moves, 4, KING);
    else if (input[0]=='Q')
      searchList(&moves, 4, QUEEN);
    else if (input[0]=='N')
      searchList(&moves, 4, KNIGHT);
    else if (input[0]=='B')
      searchList(&moves, 4, BISHOP);
    else if (input[0]=='R')
      searchList(&moves, 4, ROOK);
    strcpy(input,&(input[1]));     /* remove this character */
    }
  else if (islower(input[0]))
    searchList(&moves, 4, PAWN);
  else
    return mov;       /* invalid algebraic move */
 
  //printf("%d moves left after piece check\n",moves.size);

  /* 2. Find target column 
   * there are 3 cases: [row indic][Square], [col indic][square] or just [square].
   */
  if (isdigit(input[1])) tarcol = 0;
  else tarcol = 1;  

  if (input[tarcol] == 'a')
    searchList(&moves, 1, 0);
  else if (input[tarcol] == 'b')
    searchList(&moves, 1, 1);
  else if (input[tarcol] == 'c')
    searchList(&moves, 1, 2);
  else if (input[tarcol] == 'd')
    searchList(&moves, 1, 3);
  else if (input[tarcol] == 'e')
    searchList(&moves, 1, 4);
  else if (input[tarcol] == 'f')
    searchList(&moves, 1, 5);
  else if (input[tarcol] == 'g')
    searchList(&moves, 1, 6);
  else if (input[tarcol] == 'h')
    searchList(&moves, 1, 7);

  //printf("%d moves left after column check\n",moves.size); 

  /* 3. Find target row */
  tarcol++;
  searchList(&moves, 0, atoi(&(input[tarcol]))-1);

  //printf("%d moves left after row check\n",moves.size);

  /* 4. Consider from row/column indicator (if it exists) */
  if (tarcol>1)
    {
    if (isdigit(input[0]))     /* row indicator */
      searchList(&moves, 2, atoi(&(input[0]))-1);
    else                      /* column indicator */
      {
      if (input[0]=='a') searchList(&moves, 3, 0);
      else if (input[0]=='b') searchList(&moves, 3, 1);
      else if (input[0]=='c') searchList(&moves, 3, 2);
      else if (input[0]=='d') searchList(&moves, 3, 3);
      else if (input[0]=='e') searchList(&moves, 3, 4);
      else if (input[0]=='f') searchList(&moves, 3, 5);
      else if (input[0]=='g') searchList(&moves, 3, 6);
      else if (input[0]=='h') searchList(&moves, 3, 7);
      }
    }

  //printf("%d moves left after from square check\n",moves.size);

  /* 5. Check for the promotion indicator */
  if (strlen(input)>tarcol+2)
    if (input[tarcol+1]=='=')
      {
      if (input[tarcol+2]=='Q') searchList(&moves, 5, QUEEN);
      else if (input[tarcol+2]=='R') searchList(&moves, 5, ROOK);
      else if (input[tarcol+2]=='B') searchList(&moves, 5, BISHOP);
      else if (input[tarcol+2]=='N') searchList(&moves, 5, KNIGHT);
      }

  //printf("%d moves left after promotion check\n",moves.size);

  /* Now we check how many moves we have left and return either
   * the correct move or an error move appropriately
   */
  if (moves.size==1)
     moveCopy(getMove(moves) ,&mov);
  else if (moves.size==0)
     mov.from_row=-1;
  else
     mov.from_row=-2;
     
  clearList(&moves);
  return mov;
  }

/* fills up the string 'status' with either '+',
 * '#' or '', depending on what the given move
 * leads to.
 */
void findStatus(struct Move mov, char *status)
  {
  makeMove(&mov);
  if (check_for_check(pos.who_moves))
    {
    if (has_move())
      strcpy(status, "+");
    else
      strcpy(status, "#");
    }
  else
    strcpy(status, "");

  unmakeMove(mov);
  }

/* Takes a move in the internal format and fills the 
 * string 'move' with the equivilent move in Algebraic
 * notation. It is imperitive that the board be setup
 * as the position which this move is made out of.
 */
void createAlgebraOutput(struct Move mov, char *move)
  {
  struct Move tempmove;  /* for testing the string */
  char piece[3];
  char square[3];
  char capture[3];
  char status[3];
  char promote[3];
  char indicator[3];

  /* The special castle cases come first */
  if (mov.from_row==0&&mov.from_column==4&&mov.to_row==0&&mov.to_column==6)
    if (pos.board[mov.from_row][mov.from_column].piece==KING)
      {strcpy(move, "O-O"); return;}
  if (mov.from_row==7&&mov.from_column==4&&mov.to_row==7&&mov.to_column==6)
    if (pos.board[mov.from_row][mov.from_column].piece==KING)
      {strcpy(move, "O-O"); return;}
 if (mov.from_row==0&&mov.from_column==4&&mov.to_row==0&&mov.to_column==2)
    if (pos.board[mov.from_row][mov.from_column].piece==KING)
      {strcpy(move, "O-O-O"); return;}
 if (mov.from_row==7&&mov.from_column==4&&mov.to_row==7&&mov.to_column==2)
    if (pos.board[mov.from_row][mov.from_column].piece==KING)
      {strcpy(move, "O-O-O"); return;}

  /* Set the piece type indicator */
  if (pos.board[mov.from_row][mov.from_column].piece == KING)
    strcpy(piece,"K");
  else if (pos.board[mov.from_row][mov.from_column].piece == QUEEN)
    strcpy(piece,"Q");
  else if (pos.board[mov.from_row][mov.from_column].piece == KNIGHT)
    strcpy(piece,"N");
  else if (pos.board[mov.from_row][mov.from_column].piece == BISHOP)
    strcpy(piece,"B");
  else if (pos.board[mov.from_row][mov.from_column].piece == ROOK)
    strcpy(piece,"R");
  else
    strcpy(piece,"");

  /* Set the to-square indicator */
  square[0] = 97+mov.to_column;
  square[1] = 49+mov.to_row;
  square[2] = '\0';

  /* Set the capture indicator */
  if (mov.piece != EMPTY) strcpy(capture,"x");
  else strcpy(capture,"");

  /* If capturer was a pawn,.. we must indicate the from column */
  if (mov.piece && pos.board[mov.from_row][mov.from_column].piece==PAWN)
    { piece[0]=97+mov.from_column; piece[1]='\0';}
  
  /* Now add the + or # for check or mate */
  findStatus(mov, status);

  /* Now figure out the promotion indicator */
  if (mov.promote)
    {
    promote[0]='=';
    if (mov.promote==QUEEN)
      promote[1]='Q';
    else if (mov.promote==ROOK)
      promote[1]='R';
    else if (mov.promote==KNIGHT)
      promote[1]='N';
    else if (mov.promote==BISHOP)
      promote[1]='B';
    promote[2]='\0';
    }
  else
    strcpy(promote, "");

  /* Now construct the string and test it */
  strcpy(move, piece); 
  strcat(move, capture); 
  strcat(move, square);
  strcat(move, promote);
  strcat(move, status);

  moveCopy(parseAlgebra(move), &tempmove);
  
  /* If the returned move is invalid, we need to add an ambiguity indicator */

  if (tempmove.from_row>=0)
    return; 

  /* Start by adding the column */
  indicator[0]=97+mov.from_column;
  indicator[1]='\0';
  strcpy(move, piece);
  strcat(move, indicator);
  strcat(move, capture);
  strcat(move, square);
  strcat(move, promote);
  strcat(move, status);
  moveCopy(parseAlgebra(move), &tempmove);

  if (tempmove.from_row>=0)
    return;

  /* Now try adding the row */
  indicator[0]=49+mov.from_row;
  indicator[1]='\0';
  strcpy(move, piece);
  strcat(move, indicator);
  strcat(move, capture);
  strcat(move, square);
  strcat(move, promote);
  strcat(move, status);
  
  return;    /* We must be left with a good move,.. because there are no other options! */  
  }
  


void printHelp (void)
{
printf("%s\nChess Engine - code and design by Alex Boby\n",ProgramName);
printf("Use \"helpargs\" for a list of command line arguments and program parameters\n");
printf("Use \"helpcoms\" for a list of program commands\n");
}

/* This prints the help info when the program is called with 
 * the 'help' argument
 */
void printHelpArguments (void)
{
printf("%s\nChess Engine - code and design by Alex Boby\n",ProgramName);
printf("Command Line Arguments: [options or defaults in brackets]\n");
printf("program strength:\n");    
printf("     depth [1-10]             sets the search depth [0] *currently disabled - use searchtime*\n");
printf("     qdepth [#]               sets the quiescence search depth [0] *currently disabled*\n");
printf("     epsilon [#]              sets the apsiration search window size [50]\n");
printf("     searchtime [#]           sets the searchtime per move in milliseconds [auto]\n");
printf("opening book:\n");
printf("     nobook                   disables the opening book\n");
printf("     bookfile [filename]      specify the opening book file [openings.pbf]\n");
printf("     makebook                 creates a new book file (overwrites the old one)\n");
printf("                              with the name specified by bookfile\n");
printf("     buildpgn [filename] [#]  builds (or extends) the opening book by adding the first # moves\n");
printf("                              from each game in filename\n");
printf("i/o, tests:\n");
printf("     fenfile [filename]       loads the first FEN position in the file as the starting position\n");
printf("     fentest [filename]       runs a test on FEN posiitons. File format is FEN position and solution\n");
printf("                              in algebraic notation on alternating lines\n");
printf("     readpgn [filename]       reads in the first game in a PGN file\n");
printf("     loggames [filename]      logs all games in PGN format to filename upon receipt of the result command\n");
printf("debugging:\n");
printf("     draw                     enables draw mode - the board will be drawn\n");
printf("                              in ascii text to stdout after each move\n");
printf("     debug                    prints lots of debugging info to stdout\n");
printf("     stats                    prints the search stats after each tree search\n");
printf("     whisper                  sets Pierre to output some search stats to ICS\n");
printf("                              after each move search\n");
printf("     showthink                enables verbose output while thinking about moves\n");
}

void printHelpCommands(void)
{
printf("%s\nChess Engine - code and design by Alex Boby\n", ProgramName);
printf("Program Commands: (fully xboard/winboard compliant)\n");
printf("playing:\n");
printf("     new                   begins a new game\n");
printf("     [let][#][let][#]      user move - eg e2e4 (coordinate notation)\n");
printf("     [xxx]                 user move - eg Bxc4 (algebraic notation)\n");
printf("     white                 sets Pierre to play white\n");
printf("     black                 sets Pierre to play black\n");
printf("     force                 sets Pierre to play neither colour\n");
printf("     undo                  takes back the last move\n");
printf("     go,beep,random        null - xboard commands\n");
printf("     xboard,hard           null - xboard commands\n");
printf("     draw                  request a drawn game\n");
printf("     drawboard             draws the current position to stdout\n");
printf("     post, nopost          toggles \"show thinking\" under xboard\n");
printf("     searchtime [#]        sets the searchtime per move in milliseconds\n");
printf("     quit,exit             exit program\n");
printf("ICS commands:\n");
printf("     name [str]            sets the opponents name\n");
printf("     rating [#] [#]        sets ratings of players, computer then opponent.\n");
printf("     level [x] [#1] [#2]   sets the clocks to initial time #1, increment #2\n");
printf("     time,otim [#]         updates the clocks to # centiseconds remaining\n");
printf("     result [string]       confirm the result of the game\n");
printf("opening book:\n");
printf("     showbook              shows all the book moves out of the current position\n");
printf("     updatebook            updates the book file with the games history\n");
printf("debugging:\n");
printf("     book                  returns true if the computer will search the\n");
printf("                           opening book for its next move\n");
printf("     debugflags            shows player enpassant flags\n");
printf("     showbit               prints the dynamic bitboards to stdout\n");
printf("     testbit               prints the list of moves available to stdout\n");
printf("     compboards            returns true if the matrix and the bitboard\n");
printf("                           positions are the same\n");
printf("     checksum              prints the current position checksum\n");
printf("     rep                   gives the number of times the current position\n");
printf("                           has been repeated in the current game\n");
printf("     testalg [move]        takes a move in algebraic notation and outputs\n");
printf("                           its interpretation of the move in coordinate notation\n");
printf("     savegame              saves current game to the file specified by loggames\n");
printf("     hint                  pretends its the computers turn\n");
printf("     analysis              think about the move but do not play. Set searchtime to\n");
printf("                           zero for an indefinate analysis\n");
}

/* This function ensures that there is always an open file for status output.
 * Should be called at the beginning of each new game. Returns a valid file pointer.
 */
void statusFile (FILE **fp)
  {
  int i;  
  char filename[20];

  if(*fp!=NULL)fclose(*fp);
         
  /* now look for a file which doesn't already exist */
  for (i=1; i<10000; i++)
    {
    strcpy(filename, "status.");
    strcat(filename, (char *)i);
    if (!access(filename, F_OK))
      break;
    }
  *fp = fopen(filename, "wt");
  }

/* This function handles the general status message output. It is responsible
 * for the ICS status whispers as well as local message logging file(s)
 */
void messageOutput(int movenum, char *move, float time, float nps, int book, int score, int rep)
  {
  printf("tellics whisper %d:%s T:%2.2fs NPS:%.0f B:%d S:%d R:%d\n",movenum,move,time,nps, book, score, rep); 
  fflush(stdout);
  }

/* Handles the opening and closing of the console log file(s). One log file is 
 * created for every session. Should be called upon program execution to open a
 * non-existing file. Returns false if the file could not be opened.
 */
int consoleFile (FILE **fp)
  {
  int i;
  char filename[20];

  for (i=48;i<126;i++)
    {
    strcpy(filename,"console.");
    strcat(filename, (char *)&i);
    if (access(filename, 0))
      break;
    }
  if ((*fp = fopen(filename, "wt"))==NULL)
    return FALSE;
  return TRUE;
  }
  
/* this function is responsible for logging all console input and output to the 
 * console log file. inout takes value 0 for input and 1 for output
 */
void logConsole (char *msg, int inout, FILE **fp)
  {
  if (CONSOLELOG)
    {
    if (inout == 1)
      fprintf(*fp, "out: %s\n",msg);
    else if (inout == 0)
      fprintf(*fp, "in: %s\n",msg);
    }
  }

/* this functions handles the general output for the program. It echos the msg input to
 * stdout and then also echos the input to the console log if console logging is enabled.
 */
void outp(char *msg, FILE **fp)
  {
  printf(msg);
  fflush(stdout);
  
  if (CONSOLELOG)
    logConsole (msg, 1, fp);
  }

void printSearchStats (void)
  {
  double timeSearching = (double)(searchEndTime-searchStartTime)/(double)1000;
  printf("------------- Search Stats ---------------\n");
  printf("Depth Searched:   %d\n",DEPTH);
  printf("Total Time:       %2.3fs\n",timeSearching);
  printf("Moves Generated:  %ld\n",numgenerated);
  printf("Extra Moves Gd:   %ld\n",extragenerated);
  printf("Time Generating:  %2.3fs [%d%% of total]\n",timeGenerating, (int)((timeGenerating/timeSearching)*100));
  printf("Generation Speed: %ld moves/sec\n",(long)((double)(numgenerated+extragenerated)/timeGenerating));
  printf("Positions Evald:  %ld\n",numevaluated);
  printf("Time Evaluating:  %2.3fs [%d%% of total]\n",timeEvaluating, (int)((timeEvaluating/timeSearching)*100));
  printf("Eval Speed:       %ld pos/sec\n",(long)((double)numevaluated/timeEvaluating));
  printf("Moves Considered: %ld\n",nummoves);
  printf("Quienscence Lns:  %ld\n",extralines);
  printf("------------- Eval Times -----------------\n");
  printf("Mate:             %2.3fs\n",time1);
  printf("Repetition:       %2.3fs\n",time2);
  printf("Doubled Pawns:    %2.3fs\n",time3);
  printf("Castling Elig:    %2.3fs\n",time4);
  printf("------------------------------------------\n");
  }

/* Mode = 1 for normal or = 0 for book move */
void printWhisper (int mode, int score)
  {
  double timeSearching = (double)(searchEndTime-searchStartTime)/(double)1000;
  
  if (mode==1)
    {
    printf("tellics whisper Time: %.2lf Depth: %d Score: %+.2f PV: ",timeSearching, DEPTH, (float)(score-initialEval)/100);
    printPV(realPV);
    printf("\n");
    }
  }

void moveBack(int x)
  {
  int i;

  for (i=0;i<x;i++)
    printf("\b");
  }

void clearLine(int x)
  {
  int i;

  for (i=0;i<x;i++)
    printf(" ");
  moveBack(x);
  }








