/* Pierre's Lists Source File
 * ----> These are the functions which maintain linked lists.
 * These lists are used by pierre to store lists of possible moves used
 * in the tree search and to store the game's move history. The Book Lists
 * are used to store possible book moves out of a position. The Checksum
 * list is used for repetition detection.
 */


/* ******* General Move List Routines for Book Move List's ******** */
/* **************************************************************** */
/* **************************************************************** */

/* initialize a Book Move list by setting it's size to zero */
void initBookList (struct BookMoveList *bml)
  {
  bml->size = 0;
  }

/* add's a Book move to the beginning of a Book Move list */
void addBookMove (struct BookMove mov, struct BookMoveList *bml)
  {
  FILE *fp;
  char errormessage [80]= "Error allocating memory in addBookMove";
  struct BookMoveListNode *newnode;    /* the new book list node */
  if((newnode = (struct BookMoveListNode *) malloc (sizeof(struct BookMoveListNode)))==NULL)
    {
    printf("*** ERROR *** OUT OF MEMORY\n");
    /* Make an error log */
    fp = fopen("perror.log","wb+");
    fwrite(errormessage, sizeof(char[80]), 1, fp);
    fclose(fp);
    exit(1);
    }
  bookMoveCopy (mov, &(newnode->bmov));
  if (bml->size != 0)
    newnode->next = bml->head;
  bml->head = newnode;
  if (bml->size == 0)
    bml->tail = newnode;
  bml->size++;
  }

/*   Add's a Bookmove to the end of a Book Move list  */
void appendBookMove (struct BookMove mov, struct BookMoveList *bml)
  {
  FILE *fp;
  char errormessage[80] = "Error allocating memory in appendBookMove";
  struct BookMoveListNode *current;
  struct BookMoveListNode *newnode;          /* the new list node */
  if((newnode = (struct BookMoveListNode *) malloc (sizeof(struct BookMoveListNode)))==NULL)
    {
    printf("*** ERROR *** OUT OF MEMORY\n");
    /* Make an error log */
    fp = fopen("perror.log","wb+");
    fwrite(errormessage, sizeof(char[80]), 1, fp);
    fclose(fp);
    exit(1);
    }
  bookMoveCopy (mov, &(newnode->bmov));
  if (bml->size == 0)
    bml->head = newnode;
  else
    {
    current = bml->tail;    /* find the last node */
    current->next = newnode;
    }
  bml->tail = newnode;
  bml->size++;
  }

/* copies the contents of one Book list into the other */
void bookListCopy (struct BookMoveList bml1, struct BookMoveList *bml2)
  {
  int i;
  struct BookMoveListNode *current;

  bml2->size = 0;
  current = bml1.head;
  for (i=0;i<bml1.size;i++)
    {
    appendBookMove(current->bmov, bml2);
    if (i != bml1.size-1)
      current = current->next;
    }
  }

/* deletes the first move on the book list */
void deleteBookMove (struct BookMoveList *bml)
  {
  struct BookMoveListNode *current;
  struct BookMoveListNode *first;

  if (bml->size > 0)
    {
    current = bml->head;
    if (bml->size > 1)
      first = current->next;
    free(current);
    if (bml->size > 1)
      bml->head = first;
    bml->size--;
    }
  }


/* deletes all the elements in a book list */
void clearBookList (struct BookMoveList *bml)
  {
  while (bml->size != 0)
    deleteBookMove (bml);
  }

/* returns the first move on the book list */
struct BookMove getBookMove (struct BookMoveList bml)
  {
  struct BookMove themove;
  struct BookMoveListNode *current;

  current = bml.head;
  bookMoveCopy(current->bmov, &themove);

  return themove;
  }



/* ******* General Move List Routines for Normal Move Lists ******* */
/* **************************************************************** */
/* **************************************************************** */

/* initialize the list by setting it's size to zero */
void initList (struct MoveList *ml)
  {
  ml->size = 0;
  }

/* add's a move to the beginning of the list */
void addMove (struct Move mov, struct MoveList *ml)
  {
  FILE *fp;
  char errormessage [80]= "Error allocating memory in addMove";
  struct MoveListNode *newnode;    /* the new list node */
  if((newnode = (struct MoveListNode *) malloc (sizeof(struct MoveListNode)))==NULL)
    {
    printf("*** ERROR *** OUT OF MEMORY\n");
    /* Make an error log */
    fp = fopen("perror.log","wb+");
    fwrite(errormessage, sizeof(char[80]), 1, fp);
    fclose(fp);
    exit(1);
    }
  moveCopy (mov, &(newnode->move));
  if (ml->size != 0)
    newnode->next = ml->head;
  ml->head = newnode;
  if (ml->size == 0)
    ml->tail = newnode;
  ml->size++;
  }

/*   Add's a move to the end of the list
 *   (needed to facilitate list copying)
 */
void appendMove (struct Move mov, struct MoveList *ml)
  {
  FILE *fp;
  char errormessage[80] = "Error allocating memory in appendMove";
  struct MoveListNode *current;
  struct MoveListNode *newnode;          /* the new list node */
  if((newnode = (struct MoveListNode *) malloc (sizeof(struct MoveListNode)))==NULL)
    {
    printf("*** ERROR *** OUT OF MEMORY\n");
    /* Make an error log */
    fp = fopen("perror.log","wb+");
    fwrite(errormessage, sizeof(char[80]), 1, fp);
    fclose(fp);
    exit(1);
    }
  moveCopy (mov, &(newnode->move));
  if (ml->size == 0)
    ml->head = newnode;
  else
    {
    current = ml->tail;    /* find the last node */
    current->next = newnode;
    }
  ml->tail = newnode;
  ml->size++;
  }

/* copies the contents of one list into the other */
void listCopy (struct MoveList ml1, struct MoveList *ml2)
  {
  int i;
  struct MoveListNode *current;

  ml2->size = 0;
  current = ml1.head;
  for (i=0;i<ml1.size;i++)
    {
    appendMove(current->move, ml2);
    if (i != ml1.size-1)
      current = current->next;
    }
  }

/* deletes the first move on the list */
void deleteMove (struct MoveList *ml)
  {
  struct MoveListNode *current;
  struct MoveListNode *first;

  if (ml->size > 0)
    {
    current = ml->head;
    if (ml->size > 1)
      first = current->next;
    free(current);
    if (ml->size > 1)
      ml->head = first;
    ml->size--;
    }
  }

/* deletes all the elements in a list */
void clearList (struct MoveList *ml)
  {
  while (ml->size != 0)
    deleteMove (ml);
  }

/* returns the first move on the list */
struct Move getMove (struct MoveList ml)
  {
  struct Move themove;
  struct MoveListNode *current;

  current = ml.head;
  moveCopy(current->move, &themove);

  return themove;
  }

/* Routine takes a full list m1 and an empty list m2. It reverses the
 * order of the contents in m1 and stores them in m2. This routine is
 * is needed because the book search needs to have the moves in order,
 * whereas the history needs to be stored in reverse order.
 */
void reverseList (struct MoveList m1, struct MoveList *m2)
  {
  struct MoveList templist;
  struct Move tempmove;

  /* first make a copy of list m1 */
  listCopy(m1, &templist);

  clearList(m2);
  while (templist.size)
    {
    moveCopy(getMove(templist), &tempmove);
    addMove(tempmove, m2);
    deleteMove(&templist);
    }
  }

/* Deletes all the moves from m1 except for the first 'num'.
 * Needed to update the book from PGN's with only a part of
 * the game's move history. If there are less than 'num' moves
 * in the list,.. nothing is done.
 */
void getPartialList(struct MoveList *ml, int num)
  {
  while (ml->size > num)
    deleteMove(ml);
  }

/* searchList is used to search through the list and delete the
 * moves which don't meet the given criteria. 
 * The types are 0 to_row, 1 to_column, 2 from_row, 3 from_column,
 * 4 piece and 5 promote.
 * This function is needed for the algebraic notation routines
 */
void searchList (struct MoveList *ml, int type, int value)
  {
  struct MoveListNode *current = ml->head;
  struct MoveListNode *previous = NULL;

  while (previous!=ml->tail)
    {
    if (((type==0) && (current->move.to_row!=value)) || ((type==1)&&(current->move.to_column!=value)) || ((type==2) && (current->move.from_row!=value)) || ((type==3)&&(current->move.from_column!=value)) || ((type==4)&&(pos.board[current->move.from_row][current->move.from_column].piece!=value)) || ((type==5)&&(current->move.promote!=value)))
      {
      /* delete this node, move does not match criteria */
      if (previous == NULL)       /* first node */      
        ml->head = current->next;
      else
        previous->next = current->next;
      free(current);
      ml->size--;
      /* need to update the tail if the last move has been removed */
      if (current == ml->tail)
        ml->tail=previous;
      if (previous == NULL)
        current = ml->head;
      else
        current = previous->next;
      }
    else
      {
      previous = current;
      current = current->next;
      }
    }
  }

/* Prints the contents of the list and removes all elements */
void printMoveList (struct MoveList *ml)
  {
  struct Move tempmove;
  char output[10];

  while (ml->size)
    {
    moveCopy(getMove(*ml), &tempmove);
    createMoveOutput(tempmove, output);
    printf("[%s] piece: %d promote: %d\n",output, tempmove.piece, tempmove.promote); 
    deleteMove(ml);
    }
  }

/* Prints the contents of the list in PV style. Contents remain
 * unchanged. It is necessary that the first move to be displayed
 * will be made out of the current board position in order for the 
 * algebraic notation output to work properly. List should have first
 * move to be played on the top.
 */
void printPV (struct MoveList ml)
  {
  struct Move tempmove;
  struct MoveList templist;
  char output[10];

  initList(&templist);
  listCopy(ml, &templist);

  if (!templist.size)
    { 
    printf("no moves\n");
    if (FENTEST)
      fprintf(fentestfile, "no moves\n");
    }
  if (pos.movenum%2==0 && templist.size)
    {
    printf("%d. ...", pos.movenum/2);
    if (FENTEST)
      fprintf(fentestfile,"%d. ...", pos.movenum/2);
    }

  while(templist.size)
    {
    moveCopy(getMove(templist), &tempmove);
    createAlgebraOutput(tempmove, output);
    //createMoveOutput(tempmove, output);
    if (pos.movenum%2==1)
      {
      if (templist.size != ml.size)
	{
        printf(" %d.",pos.movenum/2+1);
        if (FENTEST)
          fprintf(fentestfile, " %d.",pos.movenum/2+1);
	}
      else
	{
        printf("%d.",pos.movenum/2+1);
        if (FENTEST)
          fprintf(fentestfile, "%d.",pos.movenum/2+1);
	}
      }
    printf(" %s",output);
    if (FENTEST) fprintf(fentestfile, " %s",output);
    makeMove(&tempmove);
    deleteMove(&templist);
    }
  printf("\n");  
  if (FENTEST) fprintf(fentestfile, "\n");

  /* Now we need to unmake all the moves so that the position
   * is restored in it's original state.
   */
  reverseList(ml, &templist);
  while (templist.size)
    {
    moveCopy(getMove(templist), &tempmove);
    unmakeMove(tempmove);
    deleteMove(&templist);
    }
  }

/* Stores the depth PV by combining the newMove with the
 * PV from DEPTH-1.
 */
void storePV(struct Move newMove, int depth)
  {
  if (depth==1)
    {
    clearList(&(pv[DEPTH-1]));
    addMove(newMove, &(pv[DEPTH-1]));
    return;
    }
 
  clearList(&(pv[DEPTH-depth]));
  listCopy(pv[DEPTH-depth+1],&(pv[DEPTH-depth]));
  addMove(newMove, &(pv[DEPTH-depth]));
  return;
  }

/* Puts mov onto the front of ml. Used during Principle
 * variation search.
 */
void sortList(struct Move mov, struct MoveList *ml)
  {
  struct MoveList templist;
  struct Move tempmove;

  listCopy(*ml, &templist);
  clearList(ml);

  while(templist.size)
    {
    moveCopy(getMove(templist),&tempmove);
    deleteMove(&templist);
    if (moveCompare(tempmove, mov))
      addMove(tempmove, ml);
    else
      appendMove(tempmove,ml);
    }
  }

  
/* ******* General List Routines for Checksum Lists ******* */
/* ******************************************************** */
/* ******************************************************** */

/* initialize the list by setting it's size to zero */
void initChecksumList (struct ChecksumList *csl)
  {
  csl->size = 0;
  }

/* copies the global checksum string into the data field supplied */
void copyChecksum (char *st)
  {
  int i;
  
  for (i=0;i<32;i++)
    st[i] = posChecksum[i];
  }

/* adds the checksum for the current position  to the beginning of the list */
void addChecksum (struct ChecksumList *csl)
  {
  FILE *fp;
  char errormessage [80]= "Error allocating memory in addChecksum";
  struct Checksum *newnode;    /* the new list node */
  if((newnode = (struct Checksum *) malloc (sizeof(struct Checksum)))==NULL)
    {
    printf("*** ERROR *** OUT OF MEMORY\n");
    /* Make an error log */
    fp = fopen("perror.log","wb+");
    fwrite(errormessage, sizeof(char[80]), 1, fp);
    fclose(fp);
    exit(1);
    }
  copyChecksum (newnode->data);
  newnode->next = csl->head;  
  csl->head = newnode;
  csl->size++;
  }

/* deletes the first move on the list */
void deleteChecksum (struct ChecksumList *csl)
  {
  struct Checksum *current;
  struct Checksum *first;

  if (csl->size > 0)
    {
    current = csl->head;
    if (csl->size > 1)
      first = current->next;
    free(current);
    if (csl->size > 1)
      csl->head = first;
    csl->size--;
    }
  }

/* deletes all the elements in a list */
void clearChecksumList (struct ChecksumList *csl)
  {
  while (csl->size != 0)
    deleteChecksum (csl);
  }

/* returns true/false depending on whether the given checksum's are the same 
 */
int checksumCompare (char *one, char *two)
  {
  int i;
  for (i=0;i<32;i++)
    if (one[i]!=two[i]) return FALSE;
  return TRUE;
  }
    
/* This function checks for 3 repetitions of the first position in the given checksum
 * list. It only checks every second one, because the same player must be on move in
 * each occurance of the repeated position. Returns true/false.
 */
int checkRepetition (struct ChecksumList *csl)
  {
  struct Checksum *first = csl->head;
  struct Checksum *current = first;
  int i, num = 1;

  for (i=1; i<csl->size; i++)
    {
    current = current->next;
    if (((i%2)==0) && (checksumCompare(first->data, current->data)))
      num++;
    }
  
  return num;
  }

















