/* Contains routines for the output and input of PGN game files */


/* Appends the current game onto the end of the PGN log file */
void logPGN (struct MoveList history)
  {
  FILE *fp;
  struct MoveList moves;
  struct Move mov;
  int num = 0;
  int i;
  int spaces;
  char output[10];
  char result[100]; 

  if (!(fp=fopen(pgnlogfile,"at")))
    {
    printf("Error opening PGN file.\n");
    return;
    }

  fprintf(fp, "\n");

  /* Print the headers from the Game info */
  fprintf(fp, "[Event \"%s\"]\n", game.event);
  fprintf(fp, "[Site \"%s\"]\n", game.site);
  fprintf(fp, "[Date \"%02d.%02d.%02d\"]\n", game.month,game.day,game.year);
  fprintf(fp, "[Time \"%d:%02d\"]\n", game.hour, game.minutes);
  fprintf(fp, "[Round \"%d\"]\n", game.round);
  fprintf(fp, "[White \"%s\"]\n", game.white);
  if (game.whiteELO!=0) fprintf(fp, "[WhiteELO \"%d\"]\n", game.whiteELO);
  fprintf(fp, "[Black \"%s\"]\n", game.black);
  if (game.blackELO!=0) fprintf(fp, "[BlackELO \"%d\"]\n", game.blackELO);
  if (game.numMoves) fprintf(fp, "[TimeControl \"%ld/%d\"]\n", game.gameTime/60000, game.numMoves);
  else fprintf(fp, "[TimeControl \"%ld %ld\"]\n",game.gameTime/60000, game.gameInc/1000);
  fprintf(fp, "[Result \"%s\"]\n", game.result);
    
  /* The history list is more like a stack,.. in that the last move made
   * is stored at the front and the first move made is at the end... We 
   * need this list in the opposite order...
   */

  initList(&moves);
  reverseList(history, &moves);

  /* Now we are ready to write the moves section... */
  
  fprintf(fp, "\n");
  initBoard();
  initBitboardPositions();
  initPiecescores();

  num=moves.size;
  spaces = 0;

  for (i=0;i<num;i++)
    {
    if (i%2==0) fprintf(fp, "%d. ",(i+2)/2);
    moveCopy(getMove(moves),&mov);
    createAlgebraOutput(mov, output);
    fprintf(fp, "%s ", output);
    spaces+=strlen(output);
    if (spaces>=45) {fprintf(fp, "\n"); spaces=0;}
    makeMove(&mov);
    deleteMove(&moves);
    }

  strcpy(result, game.result);
  parseResult(result);
  fprintf(fp, " %s\n",result);
  clearList(&moves);
  fclose(fp);
  }

int isTag(char *stuff)
  {
  if (stuff[0]=='[')
    return TRUE;
  return FALSE;
  }


int isComment(char *stuff)
  {
  if (stuff[0]=='{')
    return TRUE;
  return FALSE;
  }

int isEndComment(char *stuff)
  {
  if (stuff[strlen(stuff)-1]=='}')
    return TRUE;
  return FALSE;
  } 

int isResult(char *stuff)
  {
  if (!strcmp(stuff, "0-1")) return TRUE;
  else if (!strcmp(stuff, "1-0")) return TRUE;
  else if (!strcmp(stuff, "1/2-1/2")) return TRUE;
  else return FALSE;
  }

void parsePGNtag(char *stuff)
  {
    //printf("Ignoring Tag: %s\n",stuff);
  /* we don't care about them yet! */
  }

/* Reads the next game from the given file. returns FALSE if
 * there are no more games.
 */
int readNextGame(FILE **fp, struct MoveList *moves)
  {
  char input[200];
  char moreInput[200];
  struct Move tempmove; 
 
  if (moves->size)
    clearList(moves);
  initList(moves);


  initBoard();
  initPiecescores();
  initBitboardPositions();

  while(fscanf(*fp, "%s", input)!=EOF)
    {
    if (isComment(input))
      {
      /* read till the end of the comment */
      while(fscanf(*fp, "%s", moreInput)!=EOF)
        {
	strcat(input, " ");
        strcat(input, moreInput);
        if (isEndComment(moreInput))
          {
	    //printf("Ignoring Comment: %s\n",input);
          break;
	  }
	}
      }
    else if (isTag(input))
      {
      if (fgets(moreInput, 150, *fp)==NULL) {printf("End of File 2\n");return FALSE;}
      removeNewline(moreInput);
      strcat(input, moreInput);
      parsePGNtag(input);
      }
    else
      {
      /* possibly a move */
      moveCopy(parseAlgebra(input), &tempmove);
      if (tempmove.from_row>=0)
        {
        addMove(tempmove, moves);
        makeMove(&tempmove);
        }
      else if (isResult(input))
        { 
        fgets(moreInput, 150, *fp);
        return TRUE;
        }
      }
    }
  return FALSE;
  }


int countGames(char *fileName)
  {
  FILE *fp;
  struct MoveList history;
  int count=0;

  initList(&history);

  if ((fp=fopen(fileName,"rt"))==NULL)
    {
    printf("PGN File Does not exist\n");
    return 0;
    }

  printf("Please Wait... ");
  fflush(stdout);

  while(readNextGame(&fp, &history))
    {
    count++;
    if (count%2==0) printf(".");
    else printf("*");
    fflush(stdout);
    }

  printf("\n\n");
  clearList(&history);
  fclose(fp);
  return count;
  }
 
void readPGN(char *fileName)
  {
  FILE *fp;
  struct MoveList history;
  int i=1;
  char temp[10];
  int count;

  initList(&history);

  count=countGames(fileName);

  if ((fp=fopen(fileName,"rt"))==NULL)
    {
    printf("PGN File Does not exist\n");
    return;
    }

  while(readNextGame(&fp, &history))
    {
    printf("Game #%d/%d after %d moves.\n",i,count,history.size);
    i++;
    drawBoard();
    scanf("%s",temp);
    }
 
  clearList(&history);
  fclose(fp);
  return;
  }

/* This function will build (or update) the opening book
 * from a PGN file (which may contain many games). The 'num'
 * parameter is how many moves from each game to attempt to
 * add to the book (the first 'num' moves). Returns the number
 * of moves added.
 */
int buildPGNbook(char *fileName, int num)
  {
  FILE *fp;
  struct MoveList history;
  struct MoveList tempList;
  int i=1;
  int count;
  int updatecount = 0;
  int tempnum;

  initList(&history);

  if (!BOOKMODE)
    {
    printf("*** You must be in book mode to update it's contents\n");
    return 0;
    }

  count=countGames(fileName);

  if ((fp=fopen(fileName,"rt"))==NULL)
    {
    printf("PGN File Does not exist\n");
    return 0;
    }

  while(readNextGame(&fp, &history))
    {
    printf("Game #%d/%d contains %d moves. ",i,count,history.size);
    fflush(stdout);
    listCopy(history, &tempList);
    getPartialList(&tempList, num);
    tempnum=bookUpdate(tempList);
    printf("Added %d moves.\n",tempnum); 
    i++;
    updatecount+=tempnum;
    clearList(&tempList);
    clearList(&history);
    }
 
  printf("\n\nA total of %d moves were added to the book.\n",updatecount);

  clearList(&history);
  fclose(fp);
  return updatecount;
  }









