/* These are all the functions necessary to keep track of the game
 * statistics (mainly counting valid moves)
 */



/* this function returns the # of valid moves that side to move
 * has from the given position.
 */
int count_valid_moves(void)
  {      
  int valid_moves = 0;
  struct MoveList ml;

  initList(&ml);
  generateMoves(&ml, 0);
  valid_moves = ml.size;

  clearList(&ml);

  return valid_moves;
  }

/* collects the stats (# pieces, strength and #valid moves) for each player
 * stores them in the given stats structure
 */
void collectStats(void)
  {
  int i,j;   // counters

  stats.num_black_pieces = 0;
  stats.num_white_pieces = 0;
  stats.black_strength = 0;
  stats.white_strength = 0;

  if (pos.who_moves==BLACK) 
    { 
    stats.num_black_moves = count_valid_moves();
    stats.black_in_check = check_for_check(BLACK);
    stats.white_in_check = FALSE;
    }
  else 
    {
    stats.num_white_moves = count_valid_moves();
    stats.white_in_check = check_for_check(WHITE);
    stats.black_in_check = FALSE;
    }

  for (i=0;i<8;i++)
    {
    for (j=0;j<8;j++)
      {
      if ((pos.board[i][j].colour == BLACK) && (pos.board[i][j].piece > 0))
	{
	stats.num_black_pieces++;
	if (pos.board[i][j].piece == PAWN)
	  stats.black_strength++;
	if ((pos.board[i][j].piece == KNIGHT) || (pos.board[i][j].piece == BISHOP))
	  stats.black_strength+=3;
	if (pos.board[i][j].piece == ROOK)
	  stats.black_strength+=5;
	if (pos.board[i][j].piece == QUEEN)
	  stats.black_strength+=9;
	}
      if ((pos.board[i][j].colour == WHITE) && (pos.board[i][j].piece > 0))
	{
	stats.num_white_pieces++;
	if (pos.board[i][j].piece == PAWN)
	  stats.white_strength++;
	if ((pos.board[i][j].piece == KNIGHT) || (pos.board[i][j].piece == BISHOP))
	  stats.white_strength+=3;
	if (pos.board[i][j].piece == ROOK)
	  stats.white_strength+=5;
	if (pos.board[i][j].piece == QUEEN)
	  stats.white_strength+=9;
	}
      }
    }
  }


/* Returns TRUE if the side to move has a valid move and false if he doesn't 
 * This speeds up checking for mate/stalemate during the position evaluation
 * greatly. Almost exactly the same as generateMoves() except that instead of
 * adding moves to a list, it returns TRUE as soon as it finds only 1 valid 
 * move.
 */
int has_move(void)
  {
  int i,j,k,l;
  struct Move tempmove;


  for (i=0;i<8;i++)
    {
    for (j=0;j<8;j++)
      {
      /* find all colour's pieces */
      if ((pos.board[i][j].colour == pos.who_moves) && (pos.board[i][j].piece > 0))
	{
	tempmove.from_row = i;
	tempmove.from_column = j;
	if (pos.board[i][j].piece == PAWN)
	  {
	  /* check move 1 or 2 forward and 2 killing squares */
	  if (pos.board[i][j].colour == WHITE)
	    {	   
            tempmove.to_row = i+1;       /* 1 forward */
	    tempmove.to_column = j;
	    tempmove.piece = EMPTY;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==7) tempmove.promote=QUEEN;
            if (good_move(tempmove))
              return TRUE;
	    tempmove.to_row = i+2;       /* 2 forward */
	    tempmove.to_column = j;
	    tempmove.piece = EMPTY;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==7) tempmove.promote=QUEEN;
	    if (good_move(tempmove))
	      return TRUE;
            tempmove.to_row = i+1;       /* kill to left */
	    tempmove.to_column = j-1;
	    tempmove.piece = pos.board[i+1][j-1].piece;
            if (tempmove.piece==EMPTY) tempmove.piece=ENPASSENTPAWN;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==7) tempmove.promote=QUEEN;
            if (good_move(tempmove))
              return TRUE;
	    tempmove.to_row = i+1;       /* kill to right */
	    tempmove.to_column = j+1;
	    tempmove.piece = pos.board[i+1][j+1].piece;
            if (tempmove.piece==EMPTY) tempmove.piece=ENPASSENTPAWN;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==7) tempmove.promote=QUEEN;
            if (good_move(tempmove))
              return TRUE;
	    }
	  if (pos.board[i][j].colour == BLACK)
	    {
	    tempmove.to_row = i-1;       /* 1 forward */
	    tempmove.to_column = j;
	    tempmove.piece = EMPTY;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==0) tempmove.promote=QUEEN;
            if (good_move(tempmove))
               return TRUE;
	    tempmove.to_row = i-2;       /* 2 forward */
	    tempmove.to_column = j;
	    tempmove.piece = EMPTY;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==0) tempmove.promote=QUEEN;
	    if (good_move(tempmove))
	      return TRUE;
	    tempmove.to_row = i-1;       /* kill to left */
	    tempmove.to_column = j-1;
	    tempmove.piece = pos.board[i-1][j-1].piece;
            if (tempmove.piece==EMPTY) tempmove.piece=ENPASSENTPAWN;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==0) tempmove.promote=QUEEN;
            if (good_move(tempmove))
              return TRUE;
	    tempmove.to_row = i-1;       /* kill to right */
	    tempmove.to_column = j+1;
	    tempmove.piece = pos.board[i-1][j+1].piece;
            if (tempmove.piece==EMPTY) tempmove.piece=ENPASSENTPAWN;
            tempmove.promote = EMPTY;
            if (tempmove.to_row==0) tempmove.promote=QUEEN;
            if (good_move(tempmove))
              return TRUE;
	    }
	  }
	if (pos.board[i][j].piece == ROOK)
	  {
	  /* check all verticle and all horizontal possibilities */
	  tempmove.to_row = i;
	  for (k=0;k<8;k++)   /* check horizontal (cycle columns) */
	    {
	    tempmove.to_column = k;
	    tempmove.piece = pos.board[i][k].piece;
            tempmove.promote = EMPTY;
	    if (good_move(tempmove))
	      return TRUE;
	    }
	  tempmove.to_column = j;
	  for (k=0;k<8;k++)    /* check verticle (cycle rows) */
	    {
	    tempmove.to_row = k;
	    tempmove.piece = pos.board[k][j].piece;
            tempmove.promote = EMPTY;
	    if (good_move(tempmove))
	      return TRUE;
	    }
	  }
	if (pos.board[i][j].piece == BISHOP)
	  {
	  /* check all diagonal possibilities
	   * 1. up and to the right
	   */
	  l = 7 - j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i+k][j+k].piece;
            tempmove.promote = EMPTY;
	    if (good_move(tempmove))
	      return TRUE;
	    }
	  /* 2. up and too the left */
	  l = j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i+k][j-k].piece;
            tempmove.promote = EMPTY;
	    if (good_move(tempmove))
	      return TRUE;
	    }
	  /* 3. Down and to the right */
	  l = i;
	  if (7 - j < l)
	    l = 7 - j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i-k][j+k].piece;
            tempmove.promote = EMPTY;
	    if (good_move(tempmove))
	      return TRUE;
 	    }
	  /* 4. Down and to the left */
	  l = i;
	  if (j < i)
	    l = j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i-k][j-k].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  }
	if (pos.board[i][j].piece == QUEEN)
	  {
	  /* check both rook & bishop possibilities
	   * check all verticle and all horizontal possibilities
	   */
	  tempmove.to_row = i;
	  for (k=0;k<8;k++)   /* 1. check horizontal (cycle columns) */
	    {
	    tempmove.to_column = k;
	    tempmove.piece = pos.board[i][k].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  tempmove.to_column = j;
	  for (k=0;k<8;k++)    /* check verticle (cycle rows) */
	    {
	    tempmove.to_row = k;
	    tempmove.piece = pos.board[k][j].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  /* check all diagonal possibilities
	   * 1. up and to the right
	   */
	  l = 7 - j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i+k][j+k].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  /* 2. up and too the left */
	  l = j;
	  if (7 - i < l)
	    l = 7 - i;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i+k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i+k][j-k].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  /* 3. Down and to the right */
	  l = i;
	  if (7 - j < l)
	    l = 7 - j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j+k;
	    tempmove.piece = pos.board[i-k][j+k].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  /* 4. Down and to the left */
	  l = i;
	  if (j < i)
	    l = j;
	  for (k=1;k<l+1;k++)
	    {
	    tempmove.to_row = i-k;
	    tempmove.to_column = j-k;
	    tempmove.piece = pos.board[i-k][j-k].piece;
            tempmove.promote = EMPTY;
            if (good_move(tempmove))
	      return TRUE;
	    }
	  }
	if (pos.board[i][j].piece == KNIGHT)
	  {
	  /* there are 8 possible moves */
	  tempmove.to_row = i+2;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i+2][j+1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i+2;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i+2][j-1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i+1;
	  tempmove.to_column = j+2;
	  tempmove.piece = pos.board[i+1][j+2].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i+1;
	  tempmove.to_column = j-2;
	  tempmove.piece = pos.board[i+1][j-2].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-2;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i-2][j+1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-2;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i-2][j-1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-1;
	  tempmove.to_column = j+2;
	  tempmove.piece = pos.board[i-1][j+2].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-1;
	  tempmove.to_column = j-2;
	  tempmove.piece = pos.board[i-1][j-2].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  }
	if (pos.board[i][j].piece == KING)
	  {
	  /* 10 possible moves, including the 2 castle moves */
	  tempmove.to_row = i+1;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i+1][j-1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i+1;
	  tempmove.to_column = j;
	  tempmove.piece = pos.board[i+1][j].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i+1;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i+1][j+1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i][j-1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i][j+1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-1;
	  tempmove.to_column = j-1;
	  tempmove.piece = pos.board[i-1][j-1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-1;
	  tempmove.to_column = j;
	  tempmove.piece = pos.board[i-1][j].piece;
          tempmove.promote = EMPTY;
	  if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i-1;
	  tempmove.to_column = j+1;
	  tempmove.piece = pos.board[i-1][j+1].piece;
          tempmove.promote = EMPTY;
          if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i;
	  tempmove.to_column = j+2;   /* kingside castle */
	  tempmove.piece = EMPTY;
          tempmove.promote = EMPTY;
	  if (good_move(tempmove))
	    return TRUE;
	  tempmove.to_row = i;
	  tempmove.to_column = j-2;   /* kingside castle */
	  tempmove.piece = EMPTY;
          tempmove.promote = EMPTY;
	  if (good_move(tempmove))
	    return TRUE;
	  }
	}
      }
    }
  return FALSE;
  }








